/****************************************************************************
** Meta object code from reading C++ file 'dkiminfo.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messageviewer/src/dkim-verify/dkiminfo.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dkiminfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13MessageViewer8DKIMInfoE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageViewer::DKIMInfo::qt_create_metaobjectdata<qt_meta_tag_ZN13MessageViewer8DKIMInfoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageViewer::DKIMInfo",
        "HashingAlgorithmType",
        "Any",
        "Sha1",
        "Sha256",
        "Unknown"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'HashingAlgorithmType'
        QtMocHelpers::EnumData<enum HashingAlgorithmType>(1, 1, QMC::EnumIsScoped).add({
            {    2, HashingAlgorithmType::Any },
            {    3, HashingAlgorithmType::Sha1 },
            {    4, HashingAlgorithmType::Sha256 },
            {    5, HashingAlgorithmType::Unknown },
        }),
    };
    return QtMocHelpers::metaObjectData<DKIMInfo, qt_meta_tag_ZN13MessageViewer8DKIMInfoE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageViewer::DKIMInfo::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer8DKIMInfoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer8DKIMInfoE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13MessageViewer8DKIMInfoE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
