/****************************************************************************
** Meta object code from reading C++ file 'infopart.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagecomposer/src/part/infopart.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'infopart.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15MessageComposer8InfoPartE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::InfoPart::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer8InfoPartE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::InfoPart",
        "toChanged",
        "",
        "fromChanged",
        "ccChanged",
        "bccChanged",
        "replyToChanged",
        "subjectChanged",
        "urgentChanged",
        "userAgentChanged",
        "inReplyToChanged",
        "fccChanged",
        "from",
        "to",
        "cc",
        "bcc",
        "replyTo",
        "subject",
        "fcc",
        "userAgent",
        "urgent",
        "inReplyTo"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'toChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fromChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ccChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bccChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replyToChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subjectChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'urgentChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userAgentChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inReplyToChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fccChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'from'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'to'
        QtMocHelpers::PropertyData<QStringList>(13, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'cc'
        QtMocHelpers::PropertyData<QStringList>(14, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'bcc'
        QtMocHelpers::PropertyData<QStringList>(15, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'replyTo'
        QtMocHelpers::PropertyData<QStringList>(16, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'subject'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'fcc'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'userAgent'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'urgent'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'inReplyTo'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<InfoPart, qt_meta_tag_ZN15MessageComposer8InfoPartE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::InfoPart::staticMetaObject = { {
    QMetaObject::SuperData::link<MessageComposer::MessagePart::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer8InfoPartE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer8InfoPartE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer8InfoPartE_t>.metaTypes,
    nullptr
} };

void MessageComposer::InfoPart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<InfoPart *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->toChanged(); break;
        case 1: _t->fromChanged(); break;
        case 2: _t->ccChanged(); break;
        case 3: _t->bccChanged(); break;
        case 4: _t->replyToChanged(); break;
        case 5: _t->subjectChanged(); break;
        case 6: _t->urgentChanged(); break;
        case 7: _t->userAgentChanged(); break;
        case 8: _t->inReplyToChanged(); break;
        case 9: _t->fccChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (InfoPart::*)()>(_a, &InfoPart::toChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (InfoPart::*)()>(_a, &InfoPart::fromChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (InfoPart::*)()>(_a, &InfoPart::ccChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (InfoPart::*)()>(_a, &InfoPart::bccChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (InfoPart::*)()>(_a, &InfoPart::replyToChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (InfoPart::*)()>(_a, &InfoPart::subjectChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (InfoPart::*)()>(_a, &InfoPart::urgentChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (InfoPart::*)()>(_a, &InfoPart::userAgentChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (InfoPart::*)()>(_a, &InfoPart::inReplyToChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (InfoPart::*)()>(_a, &InfoPart::fccChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->from(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->to(); break;
        case 2: *reinterpret_cast<QStringList*>(_v) = _t->cc(); break;
        case 3: *reinterpret_cast<QStringList*>(_v) = _t->bcc(); break;
        case 4: *reinterpret_cast<QStringList*>(_v) = _t->replyTo(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->subject(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->fcc(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->userAgent(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->urgent(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->inReplyTo(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFrom(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setTo(*reinterpret_cast<QStringList*>(_v)); break;
        case 2: _t->setCc(*reinterpret_cast<QStringList*>(_v)); break;
        case 3: _t->setBcc(*reinterpret_cast<QStringList*>(_v)); break;
        case 4: _t->setReplyTo(*reinterpret_cast<QStringList*>(_v)); break;
        case 5: _t->setSubject(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setFcc(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setUserAgent(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setUrgent(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setInReplyTo(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *MessageComposer::InfoPart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::InfoPart::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer8InfoPartE_t>.strings))
        return static_cast<void*>(this);
    return MessageComposer::MessagePart::qt_metacast(_clname);
}

int MessageComposer::InfoPart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MessageComposer::MessagePart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void MessageComposer::InfoPart::toChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MessageComposer::InfoPart::fromChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MessageComposer::InfoPart::ccChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MessageComposer::InfoPart::bccChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MessageComposer::InfoPart::replyToChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MessageComposer::InfoPart::subjectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void MessageComposer::InfoPart::urgentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void MessageComposer::InfoPart::userAgentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void MessageComposer::InfoPart::inReplyToChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void MessageComposer::InfoPart::fccChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
