/****************************************************************************
** Meta object code from reading C++ file 'dkimchecksignaturejob.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messageviewer/src/dkim-verify/dkimchecksignaturejob.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dkimchecksignaturejob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13MessageViewer21DKIMCheckSignatureJobE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageViewer::DKIMCheckSignatureJob::qt_create_metaobjectdata<qt_meta_tag_ZN13MessageViewer21DKIMCheckSignatureJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageViewer::DKIMCheckSignatureJob",
        "result",
        "",
        "MessageViewer::DKIMCheckSignatureJob::CheckSignatureResult",
        "checkResult",
        "storeKey",
        "key",
        "domain",
        "selector",
        "DKIMStatus",
        "Unknown",
        "Valid",
        "Invalid",
        "EmailNotSigned",
        "NeedToBeSigned",
        "DKIMError",
        "Any",
        "CorruptedBodyHash",
        "DomainNotExist",
        "MissingFrom",
        "MissingSignature",
        "InvalidQueryMethod",
        "InvalidHeaderCanonicalization",
        "InvalidBodyCanonicalization",
        "InvalidBodyHashAlgorithm",
        "InvalidSignAlgorithm",
        "PublicKeyWasRevoked",
        "SignatureTooLarge",
        "InsupportedHashAlgorithm",
        "PublicKeyTooSmall",
        "ImpossibleToVerifySignature",
        "DomainI",
        "TestKeyMode",
        "ImpossibleToDownloadKey",
        "HashAlgorithmUnsafeSha1",
        "IDomainError",
        "PublicKeyConversionError",
        "DKIMWarning",
        "SignatureExpired",
        "SignatureCreatedInFuture",
        "SignatureTooSmall",
        "HashAlgorithmUnsafe",
        "PublicRsaKeyTooSmall",
        "AuthenticationMethod",
        "Dkim",
        "Spf",
        "Dmarc",
        "Dkimatps",
        "Auth",
        "XTls",
        "Arc"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'result'
        QtMocHelpers::SignalData<void(const MessageViewer::DKIMCheckSignatureJob::CheckSignatureResult &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'storeKey'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DKIMStatus'
        QtMocHelpers::EnumData<enum DKIMStatus>(9, 9, QMC::EnumIsScoped).add({
            {   10, DKIMStatus::Unknown },
            {   11, DKIMStatus::Valid },
            {   12, DKIMStatus::Invalid },
            {   13, DKIMStatus::EmailNotSigned },
            {   14, DKIMStatus::NeedToBeSigned },
        }),
        // enum 'DKIMError'
        QtMocHelpers::EnumData<enum DKIMError>(15, 15, QMC::EnumIsScoped).add({
            {   16, DKIMError::Any },
            {   17, DKIMError::CorruptedBodyHash },
            {   18, DKIMError::DomainNotExist },
            {   19, DKIMError::MissingFrom },
            {   20, DKIMError::MissingSignature },
            {   21, DKIMError::InvalidQueryMethod },
            {   22, DKIMError::InvalidHeaderCanonicalization },
            {   23, DKIMError::InvalidBodyCanonicalization },
            {   24, DKIMError::InvalidBodyHashAlgorithm },
            {   25, DKIMError::InvalidSignAlgorithm },
            {   26, DKIMError::PublicKeyWasRevoked },
            {   27, DKIMError::SignatureTooLarge },
            {   28, DKIMError::InsupportedHashAlgorithm },
            {   29, DKIMError::PublicKeyTooSmall },
            {   30, DKIMError::ImpossibleToVerifySignature },
            {   31, DKIMError::DomainI },
            {   32, DKIMError::TestKeyMode },
            {   33, DKIMError::ImpossibleToDownloadKey },
            {   34, DKIMError::HashAlgorithmUnsafeSha1 },
            {   35, DKIMError::IDomainError },
            {   36, DKIMError::PublicKeyConversionError },
        }),
        // enum 'DKIMWarning'
        QtMocHelpers::EnumData<enum DKIMWarning>(37, 37, QMC::EnumIsScoped).add({
            {   16, DKIMWarning::Any },
            {   38, DKIMWarning::SignatureExpired },
            {   39, DKIMWarning::SignatureCreatedInFuture },
            {   40, DKIMWarning::SignatureTooSmall },
            {   41, DKIMWarning::HashAlgorithmUnsafe },
            {   42, DKIMWarning::PublicRsaKeyTooSmall },
        }),
        // enum 'AuthenticationMethod'
        QtMocHelpers::EnumData<enum AuthenticationMethod>(43, 43, QMC::EnumIsScoped).add({
            {   10, AuthenticationMethod::Unknown },
            {   44, AuthenticationMethod::Dkim },
            {   45, AuthenticationMethod::Spf },
            {   46, AuthenticationMethod::Dmarc },
            {   47, AuthenticationMethod::Dkimatps },
            {   48, AuthenticationMethod::Auth },
            {   49, AuthenticationMethod::XTls },
            {   50, AuthenticationMethod::Arc },
        }),
    };
    return QtMocHelpers::metaObjectData<DKIMCheckSignatureJob, qt_meta_tag_ZN13MessageViewer21DKIMCheckSignatureJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageViewer::DKIMCheckSignatureJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer21DKIMCheckSignatureJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer21DKIMCheckSignatureJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13MessageViewer21DKIMCheckSignatureJobE_t>.metaTypes,
    nullptr
} };

void MessageViewer::DKIMCheckSignatureJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DKIMCheckSignatureJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->result((*reinterpret_cast<std::add_pointer_t<MessageViewer::DKIMCheckSignatureJob::CheckSignatureResult>>(_a[1]))); break;
        case 1: _t->storeKey((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageViewer::DKIMCheckSignatureJob::CheckSignatureResult >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DKIMCheckSignatureJob::*)(const MessageViewer::DKIMCheckSignatureJob::CheckSignatureResult & )>(_a, &DKIMCheckSignatureJob::result, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DKIMCheckSignatureJob::*)(const QString & , const QString & , const QString & )>(_a, &DKIMCheckSignatureJob::storeKey, 1))
            return;
    }
}

const QMetaObject *MessageViewer::DKIMCheckSignatureJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageViewer::DKIMCheckSignatureJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer21DKIMCheckSignatureJobE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MessageViewer::DKIMCheckSignatureJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void MessageViewer::DKIMCheckSignatureJob::result(const MessageViewer::DKIMCheckSignatureJob::CheckSignatureResult & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MessageViewer::DKIMCheckSignatureJob::storeKey(const QString & _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
