/****************************************************************************
** Meta object code from reading C++ file 'recipientline.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagecomposer/src/recipient/recipientline.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'recipientline.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15MessageComposer17RecipientComboBoxE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::RecipientComboBox::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer17RecipientComboBoxE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::RecipientComboBox",
        "rightPressed",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'rightPressed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RecipientComboBox, qt_meta_tag_ZN15MessageComposer17RecipientComboBoxE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::RecipientComboBox::staticMetaObject = { {
    QMetaObject::SuperData::link<QComboBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer17RecipientComboBoxE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer17RecipientComboBoxE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer17RecipientComboBoxE_t>.metaTypes,
    nullptr
} };

void MessageComposer::RecipientComboBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RecipientComboBox *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->rightPressed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RecipientComboBox::*)()>(_a, &RecipientComboBox::rightPressed, 0))
            return;
    }
}

const QMetaObject *MessageComposer::RecipientComboBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::RecipientComboBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer17RecipientComboBoxE_t>.strings))
        return static_cast<void*>(this);
    return QComboBox::qt_metacast(_clname);
}

int MessageComposer::RecipientComboBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QComboBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void MessageComposer::RecipientComboBox::rightPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN15MessageComposer17RecipientLineEditE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::RecipientLineEdit::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer17RecipientLineEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::RecipientLineEdit",
        "deleteMe",
        "",
        "leftPressed",
        "rightPressed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'deleteMe'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'leftPressed'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rightPressed'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RecipientLineEdit, qt_meta_tag_ZN15MessageComposer17RecipientLineEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::RecipientLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<MessageComposer::ComposerLineEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer17RecipientLineEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer17RecipientLineEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer17RecipientLineEditE_t>.metaTypes,
    nullptr
} };

void MessageComposer::RecipientLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RecipientLineEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->deleteMe(); break;
        case 1: _t->leftPressed(); break;
        case 2: _t->rightPressed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RecipientLineEdit::*)()>(_a, &RecipientLineEdit::deleteMe, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RecipientLineEdit::*)()>(_a, &RecipientLineEdit::leftPressed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RecipientLineEdit::*)()>(_a, &RecipientLineEdit::rightPressed, 2))
            return;
    }
}

const QMetaObject *MessageComposer::RecipientLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::RecipientLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer17RecipientLineEditE_t>.strings))
        return static_cast<void*>(this);
    return MessageComposer::ComposerLineEdit::qt_metacast(_clname);
}

int MessageComposer::RecipientLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MessageComposer::ComposerLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void MessageComposer::RecipientLineEdit::deleteMe()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MessageComposer::RecipientLineEdit::leftPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MessageComposer::RecipientLineEdit::rightPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN15MessageComposer15RecipientLineNGE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::RecipientLineNG::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer15RecipientLineNGE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::RecipientLineNG",
        "typeModified",
        "",
        "MessageComposer::RecipientLineNG*",
        "addRecipient",
        "countChanged",
        "iconClicked",
        "activeChanged",
        "slotEditingFinished",
        "slotTypeModified",
        "analyzeLine"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'typeModified'
        QtMocHelpers::SignalData<void(MessageComposer::RecipientLineNG *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'addRecipient'
        QtMocHelpers::SignalData<void(MessageComposer::RecipientLineNG *, const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'iconClicked'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activeChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditingFinished'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotTypeModified'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'analyzeLine'
        QtMocHelpers::SlotData<void(const QString &)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RecipientLineNG, qt_meta_tag_ZN15MessageComposer15RecipientLineNGE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::RecipientLineNG::staticMetaObject = { {
    QMetaObject::SuperData::link<KPIM::MultiplyingLine::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer15RecipientLineNGE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer15RecipientLineNGE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer15RecipientLineNGE_t>.metaTypes,
    nullptr
} };

void MessageComposer::RecipientLineNG::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RecipientLineNG *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->typeModified((*reinterpret_cast<std::add_pointer_t<MessageComposer::RecipientLineNG*>>(_a[1]))); break;
        case 1: _t->addRecipient((*reinterpret_cast<std::add_pointer_t<MessageComposer::RecipientLineNG*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->countChanged(); break;
        case 3: _t->iconClicked(); break;
        case 4: _t->activeChanged(); break;
        case 5: _t->slotEditingFinished(); break;
        case 6: _t->slotTypeModified(); break;
        case 7: _t->analyzeLine((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageComposer::RecipientLineNG* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageComposer::RecipientLineNG* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RecipientLineNG::*)(MessageComposer::RecipientLineNG * )>(_a, &RecipientLineNG::typeModified, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RecipientLineNG::*)(MessageComposer::RecipientLineNG * , const QString & )>(_a, &RecipientLineNG::addRecipient, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RecipientLineNG::*)()>(_a, &RecipientLineNG::countChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (RecipientLineNG::*)()>(_a, &RecipientLineNG::iconClicked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (RecipientLineNG::*)()>(_a, &RecipientLineNG::activeChanged, 4))
            return;
    }
}

const QMetaObject *MessageComposer::RecipientLineNG::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::RecipientLineNG::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer15RecipientLineNGE_t>.strings))
        return static_cast<void*>(this);
    return KPIM::MultiplyingLine::qt_metacast(_clname);
}

int MessageComposer::RecipientLineNG::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPIM::MultiplyingLine::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void MessageComposer::RecipientLineNG::typeModified(MessageComposer::RecipientLineNG * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MessageComposer::RecipientLineNG::addRecipient(MessageComposer::RecipientLineNG * _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void MessageComposer::RecipientLineNG::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MessageComposer::RecipientLineNG::iconClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MessageComposer::RecipientLineNG::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
