/********************************************************************************
** Form generated from reading UI file 'sendlaterwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SENDLATERWIDGET_H
#define UI_SENDLATERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "sendlater/sendlatertimedatewidget_p.h"

QT_BEGIN_NAMESPACE

class Ui_SendLaterWidget
{
public:
    QGridLayout *gridLayout;
    QLabel *mLabel;
    MessageComposer::SendLaterTimeDateWidget *mDateTime;
    QCheckBox *mRecurrence;
    QSpinBox *mRecurrenceValue;
    QComboBox *mRecurrenceComboBox;

    void setupUi(QWidget *SendLaterWidget)
    {
        if (SendLaterWidget->objectName().isEmpty())
            SendLaterWidget->setObjectName("SendLaterWidget");
        SendLaterWidget->resize(452, 85);
        gridLayout = new QGridLayout(SendLaterWidget);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        mLabel = new QLabel(SendLaterWidget);
        mLabel->setObjectName("mLabel");

        gridLayout->addWidget(mLabel, 0, 0, 1, 1);

        mDateTime = new MessageComposer::SendLaterTimeDateWidget(SendLaterWidget);
        mDateTime->setObjectName("mDateTime");

        gridLayout->addWidget(mDateTime, 0, 1, 1, 2);

        mRecurrence = new QCheckBox(SendLaterWidget);
        mRecurrence->setObjectName("mRecurrence");

        gridLayout->addWidget(mRecurrence, 1, 0, 1, 1);

        mRecurrenceValue = new QSpinBox(SendLaterWidget);
        mRecurrenceValue->setObjectName("mRecurrenceValue");
        mRecurrenceValue->setMinimum(1);
        mRecurrenceValue->setMaximum(365);

        gridLayout->addWidget(mRecurrenceValue, 1, 1, 1, 1);

        mRecurrenceComboBox = new QComboBox(SendLaterWidget);
        mRecurrenceComboBox->setObjectName("mRecurrenceComboBox");

        gridLayout->addWidget(mRecurrenceComboBox, 1, 2, 1, 1);


        retranslateUi(SendLaterWidget);

        QMetaObject::connectSlotsByName(SendLaterWidget);
    } // setupUi

    void retranslateUi(QWidget *SendLaterWidget)
    {
        mLabel->setText(tr2i18n("Send at:", nullptr));
        mRecurrence->setText(tr2i18n("Repeat every:", nullptr));
        (void)SendLaterWidget;
    } // retranslateUi

};

namespace Ui {
    class SendLaterWidget: public Ui_SendLaterWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SENDLATERWIDGET_H

