/****************************************************************************
** Meta object code from reading C++ file 'composerviewbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagecomposer/src/composer/composerviewbase.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'composerviewbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15MessageComposer16ComposerViewBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::ComposerViewBase::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer16ComposerViewBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::ComposerViewBase",
        "sentSuccessfully",
        "",
        "Akonadi::Item::Id",
        "id",
        "failed",
        "errorMessage",
        "MessageComposer::ComposerViewBase::FailedType",
        "type",
        "modified",
        "isModified",
        "disableHtml",
        "MessageComposer::ComposerViewBase::Confirmation",
        "enableHtml",
        "tooManyRecipient",
        "identityChanged",
        "KIdentityManagementCore::Identity",
        "ident",
        "oldIdent",
        "msgCleared",
        "autoSaveMessage"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sentSuccessfully'
        QtMocHelpers::SignalData<void(Akonadi::Item::Id)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'failed'
        QtMocHelpers::SignalData<void(const QString &, MessageComposer::ComposerViewBase::FailedType)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'failed'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'modified'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'disableHtml'
        QtMocHelpers::SignalData<void(MessageComposer::ComposerViewBase::Confirmation)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'enableHtml'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tooManyRecipient'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'identityChanged'
        QtMocHelpers::SlotData<void(const KIdentityManagementCore::Identity &, const KIdentityManagementCore::Identity &, bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { 0x80000000 | 16, 18 }, { QMetaType::Bool, 19 },
        }}),
        // Slot 'identityChanged'
        QtMocHelpers::SlotData<void(const KIdentityManagementCore::Identity &, const KIdentityManagementCore::Identity &)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { 0x80000000 | 16, 18 },
        }}),
        // Slot 'autoSaveMessage'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ComposerViewBase, qt_meta_tag_ZN15MessageComposer16ComposerViewBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::ComposerViewBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer16ComposerViewBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer16ComposerViewBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer16ComposerViewBaseE_t>.metaTypes,
    nullptr
} };

void MessageComposer::ComposerViewBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ComposerViewBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sentSuccessfully((*reinterpret_cast<std::add_pointer_t<Akonadi::Item::Id>>(_a[1]))); break;
        case 1: _t->failed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MessageComposer::ComposerViewBase::FailedType>>(_a[2]))); break;
        case 2: _t->failed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->modified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->disableHtml((*reinterpret_cast<std::add_pointer_t<MessageComposer::ComposerViewBase::Confirmation>>(_a[1]))); break;
        case 5: _t->enableHtml(); break;
        case 6: _t->tooManyRecipient((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->identityChanged((*reinterpret_cast<std::add_pointer_t<KIdentityManagementCore::Identity>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KIdentityManagementCore::Identity>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 8: _t->identityChanged((*reinterpret_cast<std::add_pointer_t<KIdentityManagementCore::Identity>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KIdentityManagementCore::Identity>>(_a[2]))); break;
        case 9: _t->autoSaveMessage(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ComposerViewBase::*)(Akonadi::Item::Id )>(_a, &ComposerViewBase::sentSuccessfully, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComposerViewBase::*)(const QString & , MessageComposer::ComposerViewBase::FailedType )>(_a, &ComposerViewBase::failed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComposerViewBase::*)(bool )>(_a, &ComposerViewBase::modified, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComposerViewBase::*)(MessageComposer::ComposerViewBase::Confirmation )>(_a, &ComposerViewBase::disableHtml, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComposerViewBase::*)()>(_a, &ComposerViewBase::enableHtml, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ComposerViewBase::*)(bool )>(_a, &ComposerViewBase::tooManyRecipient, 6))
            return;
    }
}

const QMetaObject *MessageComposer::ComposerViewBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::ComposerViewBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer16ComposerViewBaseE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MessageComposer::ComposerViewBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void MessageComposer::ComposerViewBase::sentSuccessfully(Akonadi::Item::Id _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MessageComposer::ComposerViewBase::failed(const QString & _t1, MessageComposer::ComposerViewBase::FailedType _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 3
void MessageComposer::ComposerViewBase::modified(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MessageComposer::ComposerViewBase::disableHtml(MessageComposer::ComposerViewBase::Confirmation _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MessageComposer::ComposerViewBase::enableHtml()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void MessageComposer::ComposerViewBase::tooManyRecipient(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
