/********************************************************************************
** Form generated from reading UI file 'printingsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PRINTINGSETTINGS_H
#define UI_PRINTINGSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_PrintingSettings
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *mPrintEmptySelectedText;
    QCheckBox *respectExpandCollapseSettings;
    QCheckBox *printBackgroundColorAndImages;
    QCheckBox *alwaysShowEncryptionSignatureDetail;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *PrintingSettings)
    {
        if (PrintingSettings->objectName().isEmpty())
            PrintingSettings->setObjectName("PrintingSettings");
        PrintingSettings->resize(606, 348);
        verticalLayout = new QVBoxLayout(PrintingSettings);
        verticalLayout->setObjectName("verticalLayout");
        mPrintEmptySelectedText = new QCheckBox(PrintingSettings);
        mPrintEmptySelectedText->setObjectName("mPrintEmptySelectedText");

        verticalLayout->addWidget(mPrintEmptySelectedText);

        respectExpandCollapseSettings = new QCheckBox(PrintingSettings);
        respectExpandCollapseSettings->setObjectName("respectExpandCollapseSettings");

        verticalLayout->addWidget(respectExpandCollapseSettings);

        printBackgroundColorAndImages = new QCheckBox(PrintingSettings);
        printBackgroundColorAndImages->setObjectName("printBackgroundColorAndImages");

        verticalLayout->addWidget(printBackgroundColorAndImages);

        alwaysShowEncryptionSignatureDetail = new QCheckBox(PrintingSettings);
        alwaysShowEncryptionSignatureDetail->setObjectName("alwaysShowEncryptionSignatureDetail");

        verticalLayout->addWidget(alwaysShowEncryptionSignatureDetail);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(PrintingSettings);

        QMetaObject::connectSlotsByName(PrintingSettings);
    } // setupUi

    void retranslateUi(QWidget *PrintingSettings)
    {
        mPrintEmptySelectedText->setText(tr2i18n("Print only selected text", nullptr));
        respectExpandCollapseSettings->setText(tr2i18n("Respect &expand/collapse quote marks settings", nullptr));
        printBackgroundColorAndImages->setText(tr2i18n("Print background color and images", nullptr));
        alwaysShowEncryptionSignatureDetail->setText(tr2i18n("Always show encryption/signature details", nullptr));
        (void)PrintingSettings;
    } // retranslateUi

};

namespace Ui {
    class PrintingSettings: public Ui_PrintingSettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PRINTINGSETTINGS_H

