/*
   SPDX-FileCopyrightText: 2018-2026 Laurent Montel <montel@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "plugineditorconverterinitialdata.h"

using namespace MessageComposer;

PluginEditorConverterInitialData::PluginEditorConverterInitialData() = default;

bool PluginEditorConverterInitialData::newMessage() const
{
    return mNewMessage;
}

void PluginEditorConverterInitialData::setNewMessage(bool newMessage)
{
    mNewMessage = newMessage;
}

std::shared_ptr<KMime::Message> PluginEditorConverterInitialData::mewMsg() const
{
    return mMewMsg;
}

void PluginEditorConverterInitialData::setMewMsg(const std::shared_ptr<KMime::Message> &mewMsg)
{
    mMewMsg = mewMsg;
}
