/****************************************************************************
** Meta object code from reading C++ file 'mailwebengineview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messageviewer/src/viewer/webengine/mailwebengineview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mailwebengineview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13MessageViewer17MailWebEngineViewE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageViewer::MailWebEngineView::qt_create_metaobjectdata<qt_meta_tag_ZN13MessageViewer17MailWebEngineViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageViewer::MailWebEngineView",
        "wheelZoomChanged",
        "",
        "numSteps",
        "openUrl",
        "QUrl",
        "url",
        "urlBlocked",
        "messageMayBeAScam",
        "formSubmittedForbidden",
        "popupMenu",
        "WebEngineViewer::WebHitTestResult",
        "result",
        "pageIsScrolledToBottom",
        "handleScrollToAnchor",
        "QVariant",
        "handleIsScrolledToBottom",
        "slotWebHitFinished",
        "slotLoadFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'wheelZoomChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'openUrl'
        QtMocHelpers::SignalData<void(const QUrl &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'urlBlocked'
        QtMocHelpers::SignalData<void(const QUrl &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'messageMayBeAScam'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'formSubmittedForbidden'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'popupMenu'
        QtMocHelpers::SignalData<void(const WebEngineViewer::WebHitTestResult &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'pageIsScrolledToBottom'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'handleScrollToAnchor'
        QtMocHelpers::SlotData<void(const QVariant &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 12 },
        }}),
        // Slot 'handleIsScrolledToBottom'
        QtMocHelpers::SlotData<void(const QVariant &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 12 },
        }}),
        // Slot 'slotWebHitFinished'
        QtMocHelpers::SlotData<void(const WebEngineViewer::WebHitTestResult &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'slotLoadFinished'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MailWebEngineView, qt_meta_tag_ZN13MessageViewer17MailWebEngineViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageViewer::MailWebEngineView::staticMetaObject = { {
    QMetaObject::SuperData::link<WebEngineViewer::WebEngineView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer17MailWebEngineViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer17MailWebEngineViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13MessageViewer17MailWebEngineViewE_t>.metaTypes,
    nullptr
} };

void MessageViewer::MailWebEngineView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MailWebEngineView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->wheelZoomChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 2: _t->urlBlocked((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 3: _t->messageMayBeAScam(); break;
        case 4: _t->formSubmittedForbidden(); break;
        case 5: _t->popupMenu((*reinterpret_cast<std::add_pointer_t<WebEngineViewer::WebHitTestResult>>(_a[1]))); break;
        case 6: _t->pageIsScrolledToBottom((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->handleScrollToAnchor((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 8: _t->handleIsScrolledToBottom((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 9: _t->slotWebHitFinished((*reinterpret_cast<std::add_pointer_t<WebEngineViewer::WebHitTestResult>>(_a[1]))); break;
        case 10: _t->slotLoadFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MailWebEngineView::*)(int )>(_a, &MailWebEngineView::wheelZoomChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MailWebEngineView::*)(const QUrl & )>(_a, &MailWebEngineView::openUrl, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MailWebEngineView::*)(const QUrl & )>(_a, &MailWebEngineView::urlBlocked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MailWebEngineView::*)()>(_a, &MailWebEngineView::messageMayBeAScam, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MailWebEngineView::*)()>(_a, &MailWebEngineView::formSubmittedForbidden, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MailWebEngineView::*)(const WebEngineViewer::WebHitTestResult & )>(_a, &MailWebEngineView::popupMenu, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MailWebEngineView::*)(bool )>(_a, &MailWebEngineView::pageIsScrolledToBottom, 6))
            return;
    }
}

const QMetaObject *MessageViewer::MailWebEngineView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageViewer::MailWebEngineView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer17MailWebEngineViewE_t>.strings))
        return static_cast<void*>(this);
    return WebEngineViewer::WebEngineView::qt_metacast(_clname);
}

int MessageViewer::MailWebEngineView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WebEngineViewer::WebEngineView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void MessageViewer::MailWebEngineView::wheelZoomChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MessageViewer::MailWebEngineView::openUrl(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MessageViewer::MailWebEngineView::urlBlocked(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MessageViewer::MailWebEngineView::messageMayBeAScam()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MessageViewer::MailWebEngineView::formSubmittedForbidden()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MessageViewer::MailWebEngineView::popupMenu(const WebEngineViewer::WebHitTestResult & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void MessageViewer::MailWebEngineView::pageIsScrolledToBottom(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
