/****************************************************************************
** Meta object code from reading C++ file 'viewer_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messageviewer/src/viewer/viewer_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'viewer_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13MessageViewer13ViewerPrivateE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageViewer::ViewerPrivate::qt_create_metaobjectdata<qt_meta_tag_ZN13MessageViewer13ViewerPrivateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageViewer::ViewerPrivate",
        "showStatusBarMessage",
        "",
        "message",
        "popupMenu",
        "Akonadi::Item",
        "msg",
        "QUrl",
        "url",
        "imageUrl",
        "QPoint",
        "mousePos",
        "displayPopupMenu",
        "WebEngineViewer::WebHitTestResult",
        "result",
        "urlClicked",
        "requestConfigSync",
        "showReader",
        "KMime::Content*",
        "aMsgPart",
        "aHTML",
        "encoding",
        "showMessage",
        "std::shared_ptr<KMime::Message>",
        "replyMessageTo",
        "replyToAll",
        "itemRemoved",
        "makeResourceOnline",
        "MessageViewer::Viewer::ResourceOnlineMode",
        "mode",
        "changeDisplayMail",
        "MessageViewer::Viewer::DisplayFormatMessage",
        "moveMessageToTrash",
        "pageIsScrolledToBottom",
        "printingFinished",
        "zoomChanged",
        "zoomFactor",
        "showNextMessage",
        "showPreviousMessage",
        "sendResponse",
        "MessageViewer::MDNWarningWidget::ResponseType",
        "type",
        "KMime::MDN::SendingMode",
        "sendingMode",
        "slotActivatePlugin",
        "MessageViewer::ViewerPluginInterface*",
        "interface",
        "slotModifyItemDone",
        "KJob*",
        "job",
        "slotMessageMayBeAScam",
        "slotMessageIsNotAScam",
        "slotAddToWhiteList",
        "slotItemChanged",
        "item",
        "QSet<QByteArray>",
        "partIdentifiers",
        "slotItemMoved",
        "Akonadi::Collection",
        "itemModifiedResult",
        "slotClear",
        "slotMessageRendered",
        "slotOpenWithAction",
        "QAction*",
        "act",
        "slotOpenWithActionCurrentContent",
        "slotOpenWithDialog",
        "slotOpenWithDialogCurrentContent",
        "saveSplitterSizes",
        "slotRefreshMessage",
        "slotServiceUrlSelected",
        "PimCommon::ShareServiceUrlManager::ServiceType",
        "serviceType",
        "slotStyleChanged",
        "MessageViewer::HeaderStylePlugin*",
        "plugin",
        "slotStyleUpdated",
        "slotWheelZoomChanged",
        "numSteps",
        "slotToggleEmoticons",
        "slotUrlOpen",
        "slotOpenUrl",
        "slotUrlOn",
        "link",
        "slotUrlPopup",
        "slotFind",
        "slotToggleFixedFont",
        "slotToggleMimePartTree",
        "slotShowMessageSource",
        "updateReaderWin",
        "slotMimePartSelected",
        "QModelIndex",
        "index",
        "slotIconicAttachments",
        "slotSmartAttachments",
        "slotInlineAttachments",
        "slotHideAttachments",
        "slotHeaderOnlyAttachments",
        "slotDelayedResize",
        "slotPrintMessage",
        "slotPrintPreview",
        "slotSetEncoding",
        "executeCustomScriptsAfterLoading",
        "slotSettingsChanged",
        "slotMimeTreeContextMenuRequested",
        "pos",
        "slotAttachmentOpenWith",
        "slotAttachmentOpen",
        "slotAttachmentSaveAs",
        "slotAttachmentSaveAll",
        "slotAttachmentView",
        "slotAttachmentProperties",
        "slotAttachmentCopy",
        "slotAttachmentDelete",
        "slotLevelQuote",
        "l",
        "slotToggleHtmlMode",
        "slotLoadExternalReference",
        "slotHandleAttachment",
        "action",
        "slotCopySelectedText",
        "viewerSelectionChanged",
        "selectAll",
        "slotUrlCopy",
        "slotSaveMessage",
        "update",
        "MimeTreeParser::UpdateMode",
        "updateMode",
        "slotSpeakText",
        "slotCopyImageLocation",
        "slotSaveMessageDisplayFormat",
        "slotResetMessageDisplayFormat",
        "slotGeneralFontChanged",
        "slotShowDevelopmentTools"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'showStatusBarMessage'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'popupMenu'
        QtMocHelpers::SignalData<void(const Akonadi::Item &, const QUrl &, const QUrl &, const QPoint &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 7, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'displayPopupMenu'
        QtMocHelpers::SignalData<void(const Akonadi::Item &, const WebEngineViewer::WebHitTestResult &, const QPoint &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 13, 14 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'urlClicked'
        QtMocHelpers::SignalData<void(const Akonadi::Item &, const QUrl &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'requestConfigSync'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showReader'
        QtMocHelpers::SignalData<void(KMime::Content *, bool, const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { QMetaType::Bool, 20 }, { QMetaType::QString, 21 },
        }}),
        // Signal 'showMessage'
        QtMocHelpers::SignalData<void(const std::shared_ptr<KMime::Message> &, const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 3 }, { QMetaType::QString, 21 },
        }}),
        // Signal 'replyMessageTo'
        QtMocHelpers::SignalData<void(const std::shared_ptr<KMime::Message> &, bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 3 }, { QMetaType::Bool, 25 },
        }}),
        // Signal 'itemRemoved'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'makeResourceOnline'
        QtMocHelpers::SignalData<void(MessageViewer::Viewer::ResourceOnlineMode)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Signal 'changeDisplayMail'
        QtMocHelpers::SignalData<void(MessageViewer::Viewer::DisplayFormatMessage, bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 31, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Signal 'moveMessageToTrash'
        QtMocHelpers::SignalData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pageIsScrolledToBottom'
        QtMocHelpers::SignalData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'printingFinished'
        QtMocHelpers::SignalData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'zoomChanged'
        QtMocHelpers::SignalData<void(qreal)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 36 },
        }}),
        // Signal 'showNextMessage'
        QtMocHelpers::SignalData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showPreviousMessage'
        QtMocHelpers::SignalData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sendResponse'
        QtMocHelpers::SignalData<void(MessageViewer::MDNWarningWidget::ResponseType, KMime::MDN::SendingMode)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 41 }, { 0x80000000 | 42, 43 },
        }}),
        // Slot 'slotActivatePlugin'
        QtMocHelpers::SlotData<void(MessageViewer::ViewerPluginInterface *)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 45, 46 },
        }}),
        // Slot 'slotModifyItemDone'
        QtMocHelpers::SlotData<void(KJob *)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Slot 'slotMessageMayBeAScam'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMessageIsNotAScam'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddToWhiteList'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotItemChanged'
        QtMocHelpers::SlotData<void(const Akonadi::Item &, const QSet<QByteArray> &)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 54 }, { 0x80000000 | 55, 56 },
        }}),
        // Slot 'slotItemMoved'
        QtMocHelpers::SlotData<void(const Akonadi::Item &, const Akonadi::Collection &, const Akonadi::Collection &)>(57, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 2 }, { 0x80000000 | 58, 2 }, { 0x80000000 | 58, 2 },
        }}),
        // Slot 'itemModifiedResult'
        QtMocHelpers::SlotData<void(KJob *)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Slot 'slotClear'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMessageRendered'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenWithAction'
        QtMocHelpers::SlotData<void(QAction *)>(62, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 63, 64 },
        }}),
        // Slot 'slotOpenWithActionCurrentContent'
        QtMocHelpers::SlotData<void(QAction *)>(65, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 63, 64 },
        }}),
        // Slot 'slotOpenWithDialog'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenWithDialogCurrentContent'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveSplitterSizes'
        QtMocHelpers::SlotData<void() const>(68, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRefreshMessage'
        QtMocHelpers::SlotData<void(const Akonadi::Item &)>(69, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 54 },
        }}),
        // Slot 'slotServiceUrlSelected'
        QtMocHelpers::SlotData<void(PimCommon::ShareServiceUrlManager::ServiceType)>(70, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 71, 72 },
        }}),
        // Slot 'slotStyleChanged'
        QtMocHelpers::SlotData<void(MessageViewer::HeaderStylePlugin *)>(73, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 74, 75 },
        }}),
        // Slot 'slotStyleUpdated'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotWheelZoomChanged'
        QtMocHelpers::SlotData<void(int)>(77, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 78 },
        }}),
        // Slot 'slotToggleEmoticons'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUrlOpen'
        QtMocHelpers::SlotData<void(const QUrl &)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotUrlOpen'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotOpenUrl'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUrlOn'
        QtMocHelpers::SlotData<void(const QString &)>(82, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 83 },
        }}),
        // Slot 'slotUrlPopup'
        QtMocHelpers::SlotData<void(const WebEngineViewer::WebHitTestResult &)>(84, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'slotFind'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotToggleFixedFont'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotToggleMimePartTree'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowMessageSource'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateReaderWin'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMimePartSelected'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(90, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 91, 92 },
        }}),
        // Slot 'slotIconicAttachments'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSmartAttachments'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotInlineAttachments'
        QtMocHelpers::SlotData<void()>(95, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHideAttachments'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHeaderOnlyAttachments'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDelayedResize'
        QtMocHelpers::SlotData<void()>(98, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPrintMessage'
        QtMocHelpers::SlotData<void()>(99, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPrintPreview'
        QtMocHelpers::SlotData<void()>(100, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetEncoding'
        QtMocHelpers::SlotData<void()>(101, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'executeCustomScriptsAfterLoading'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMimeTreeContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(104, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 105 },
        }}),
        // Slot 'slotAttachmentOpenWith'
        QtMocHelpers::SlotData<void()>(106, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAttachmentOpen'
        QtMocHelpers::SlotData<void()>(107, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAttachmentSaveAs'
        QtMocHelpers::SlotData<void()>(108, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAttachmentSaveAll'
        QtMocHelpers::SlotData<void()>(109, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAttachmentView'
        QtMocHelpers::SlotData<void()>(110, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAttachmentProperties'
        QtMocHelpers::SlotData<void()>(111, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAttachmentCopy'
        QtMocHelpers::SlotData<void()>(112, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAttachmentDelete'
        QtMocHelpers::SlotData<void()>(113, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLevelQuote'
        QtMocHelpers::SlotData<void(int)>(114, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 115 },
        }}),
        // Slot 'slotToggleHtmlMode'
        QtMocHelpers::SlotData<void()>(116, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLoadExternalReference'
        QtMocHelpers::SlotData<void()>(117, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHandleAttachment'
        QtMocHelpers::SlotData<void(int)>(118, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 119 },
        }}),
        // Slot 'slotCopySelectedText'
        QtMocHelpers::SlotData<void()>(120, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewerSelectionChanged'
        QtMocHelpers::SlotData<void()>(121, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void()>(122, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUrlCopy'
        QtMocHelpers::SlotData<void()>(123, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSaveMessage'
        QtMocHelpers::SlotData<void()>(124, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void(MimeTreeParser::UpdateMode)>(125, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(125, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotSpeakText'
        QtMocHelpers::SlotData<void()>(128, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCopyImageLocation'
        QtMocHelpers::SlotData<void()>(129, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSaveMessageDisplayFormat'
        QtMocHelpers::SlotData<void()>(130, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotResetMessageDisplayFormat'
        QtMocHelpers::SlotData<void()>(131, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotGeneralFontChanged'
        QtMocHelpers::SlotData<void()>(132, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowDevelopmentTools'
        QtMocHelpers::SlotData<void()>(133, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ViewerPrivate, qt_meta_tag_ZN13MessageViewer13ViewerPrivateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageViewer::ViewerPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer13ViewerPrivateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer13ViewerPrivateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13MessageViewer13ViewerPrivateE_t>.metaTypes,
    nullptr
} };

void MessageViewer::ViewerPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ViewerPrivate *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showStatusBarMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->popupMenu((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[4]))); break;
        case 2: _t->displayPopupMenu((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<WebEngineViewer::WebHitTestResult>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[3]))); break;
        case 3: _t->urlClicked((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 4: _t->requestConfigSync(); break;
        case 5: _t->showReader((*reinterpret_cast<std::add_pointer_t<KMime::Content*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 6: _t->showMessage((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<KMime::Message>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->replyMessageTo((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<KMime::Message>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->itemRemoved(); break;
        case 9: _t->makeResourceOnline((*reinterpret_cast<std::add_pointer_t<MessageViewer::Viewer::ResourceOnlineMode>>(_a[1]))); break;
        case 10: _t->changeDisplayMail((*reinterpret_cast<std::add_pointer_t<MessageViewer::Viewer::DisplayFormatMessage>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->moveMessageToTrash(); break;
        case 12: _t->pageIsScrolledToBottom((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->printingFinished(); break;
        case 14: _t->zoomChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 15: _t->showNextMessage(); break;
        case 16: _t->showPreviousMessage(); break;
        case 17: _t->sendResponse((*reinterpret_cast<std::add_pointer_t<MessageViewer::MDNWarningWidget::ResponseType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KMime::MDN::SendingMode>>(_a[2]))); break;
        case 18: _t->slotActivatePlugin((*reinterpret_cast<std::add_pointer_t<MessageViewer::ViewerPluginInterface*>>(_a[1]))); break;
        case 19: _t->slotModifyItemDone((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 20: _t->slotMessageMayBeAScam(); break;
        case 21: _t->slotMessageIsNotAScam(); break;
        case 22: _t->slotAddToWhiteList(); break;
        case 23: _t->slotItemChanged((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSet<QByteArray>>>(_a[2]))); break;
        case 24: _t->slotItemMoved((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Akonadi::Collection>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Akonadi::Collection>>(_a[3]))); break;
        case 25: _t->itemModifiedResult((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 26: _t->slotClear(); break;
        case 27: _t->slotMessageRendered(); break;
        case 28: _t->slotOpenWithAction((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 29: _t->slotOpenWithActionCurrentContent((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 30: _t->slotOpenWithDialog(); break;
        case 31: _t->slotOpenWithDialogCurrentContent(); break;
        case 32: _t->saveSplitterSizes(); break;
        case 33: _t->slotRefreshMessage((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1]))); break;
        case 34: _t->slotServiceUrlSelected((*reinterpret_cast<std::add_pointer_t<PimCommon::ShareServiceUrlManager::ServiceType>>(_a[1]))); break;
        case 35: _t->slotStyleChanged((*reinterpret_cast<std::add_pointer_t<MessageViewer::HeaderStylePlugin*>>(_a[1]))); break;
        case 36: _t->slotStyleUpdated(); break;
        case 37: _t->slotWheelZoomChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 38: _t->slotToggleEmoticons(); break;
        case 39: _t->slotUrlOpen((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 40: _t->slotUrlOpen(); break;
        case 41: _t->slotOpenUrl(); break;
        case 42: _t->slotUrlOn((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 43: _t->slotUrlPopup((*reinterpret_cast<std::add_pointer_t<WebEngineViewer::WebHitTestResult>>(_a[1]))); break;
        case 44: _t->slotFind(); break;
        case 45: _t->slotToggleFixedFont(); break;
        case 46: _t->slotToggleMimePartTree(); break;
        case 47: _t->slotShowMessageSource(); break;
        case 48: _t->updateReaderWin(); break;
        case 49: _t->slotMimePartSelected((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 50: _t->slotIconicAttachments(); break;
        case 51: _t->slotSmartAttachments(); break;
        case 52: _t->slotInlineAttachments(); break;
        case 53: _t->slotHideAttachments(); break;
        case 54: _t->slotHeaderOnlyAttachments(); break;
        case 55: _t->slotDelayedResize(); break;
        case 56: _t->slotPrintMessage(); break;
        case 57: _t->slotPrintPreview(); break;
        case 58: _t->slotSetEncoding(); break;
        case 59: _t->executeCustomScriptsAfterLoading(); break;
        case 60: _t->slotSettingsChanged(); break;
        case 61: _t->slotMimeTreeContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 62: _t->slotAttachmentOpenWith(); break;
        case 63: _t->slotAttachmentOpen(); break;
        case 64: _t->slotAttachmentSaveAs(); break;
        case 65: _t->slotAttachmentSaveAll(); break;
        case 66: _t->slotAttachmentView(); break;
        case 67: _t->slotAttachmentProperties(); break;
        case 68: _t->slotAttachmentCopy(); break;
        case 69: _t->slotAttachmentDelete(); break;
        case 70: _t->slotLevelQuote((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 71: _t->slotToggleHtmlMode(); break;
        case 72: _t->slotLoadExternalReference(); break;
        case 73: _t->slotHandleAttachment((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 74: _t->slotCopySelectedText(); break;
        case 75: _t->viewerSelectionChanged(); break;
        case 76: _t->selectAll(); break;
        case 77: _t->slotUrlCopy(); break;
        case 78: _t->slotSaveMessage(); break;
        case 79: _t->update((*reinterpret_cast<std::add_pointer_t<MimeTreeParser::UpdateMode>>(_a[1]))); break;
        case 80: _t->update(); break;
        case 81: _t->slotSpeakText(); break;
        case 82: _t->slotCopyImageLocation(); break;
        case 83: _t->slotSaveMessageDisplayFormat(); break;
        case 84: _t->slotResetMessageDisplayFormat(); break;
        case 85: _t->slotGeneralFontChanged(); break;
        case 86: _t->slotShowDevelopmentTools(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KMime::Content* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< std::shared_ptr<KMime::Message> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< std::shared_ptr<KMime::Message> >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageViewer::ViewerPluginInterface* >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QByteArray> >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PimCommon::ShareServiceUrlManager::ServiceType >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(const QString & )>(_a, &ViewerPrivate::showStatusBarMessage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(const Akonadi::Item & , const QUrl & , const QUrl & , const QPoint & )>(_a, &ViewerPrivate::popupMenu, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(const Akonadi::Item & , const WebEngineViewer::WebHitTestResult & , const QPoint & )>(_a, &ViewerPrivate::displayPopupMenu, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(const Akonadi::Item & , const QUrl & )>(_a, &ViewerPrivate::urlClicked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)()>(_a, &ViewerPrivate::requestConfigSync, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(KMime::Content * , bool , const QString & )>(_a, &ViewerPrivate::showReader, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(const std::shared_ptr<KMime::Message> & , const QString & )>(_a, &ViewerPrivate::showMessage, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(const std::shared_ptr<KMime::Message> & , bool )>(_a, &ViewerPrivate::replyMessageTo, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)()>(_a, &ViewerPrivate::itemRemoved, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(MessageViewer::Viewer::ResourceOnlineMode )>(_a, &ViewerPrivate::makeResourceOnline, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(MessageViewer::Viewer::DisplayFormatMessage , bool )>(_a, &ViewerPrivate::changeDisplayMail, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)()>(_a, &ViewerPrivate::moveMessageToTrash, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(bool )>(_a, &ViewerPrivate::pageIsScrolledToBottom, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)()>(_a, &ViewerPrivate::printingFinished, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(qreal )>(_a, &ViewerPrivate::zoomChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)()>(_a, &ViewerPrivate::showNextMessage, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)()>(_a, &ViewerPrivate::showPreviousMessage, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (ViewerPrivate::*)(MessageViewer::MDNWarningWidget::ResponseType , KMime::MDN::SendingMode )>(_a, &ViewerPrivate::sendResponse, 17))
            return;
    }
}

const QMetaObject *MessageViewer::ViewerPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageViewer::ViewerPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer13ViewerPrivateE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MessageViewer::ViewerPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 87)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 87;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 87)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 87;
    }
    return _id;
}

// SIGNAL 0
void MessageViewer::ViewerPrivate::showStatusBarMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MessageViewer::ViewerPrivate::popupMenu(const Akonadi::Item & _t1, const QUrl & _t2, const QUrl & _t3, const QPoint & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 2
void MessageViewer::ViewerPrivate::displayPopupMenu(const Akonadi::Item & _t1, const WebEngineViewer::WebHitTestResult & _t2, const QPoint & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void MessageViewer::ViewerPrivate::urlClicked(const Akonadi::Item & _t1, const QUrl & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void MessageViewer::ViewerPrivate::requestConfigSync()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MessageViewer::ViewerPrivate::showReader(KMime::Content * _t1, bool _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void MessageViewer::ViewerPrivate::showMessage(const std::shared_ptr<KMime::Message> & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void MessageViewer::ViewerPrivate::replyMessageTo(const std::shared_ptr<KMime::Message> & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void MessageViewer::ViewerPrivate::itemRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void MessageViewer::ViewerPrivate::makeResourceOnline(MessageViewer::Viewer::ResourceOnlineMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void MessageViewer::ViewerPrivate::changeDisplayMail(MessageViewer::Viewer::DisplayFormatMessage _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void MessageViewer::ViewerPrivate::moveMessageToTrash()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void MessageViewer::ViewerPrivate::pageIsScrolledToBottom(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void MessageViewer::ViewerPrivate::printingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void MessageViewer::ViewerPrivate::zoomChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void MessageViewer::ViewerPrivate::showNextMessage()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void MessageViewer::ViewerPrivate::showPreviousMessage()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void MessageViewer::ViewerPrivate::sendResponse(MessageViewer::MDNWarningWidget::ResponseType _t1, KMime::MDN::SendingMode _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1, _t2);
}
QT_WARNING_POP
