/****************************************************************************
** Meta object code from reading C++ file 'recipientseditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagecomposer/src/recipient/recipientseditor.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'recipientseditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15MessageComposer20RecipientLineFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::RecipientLineFactory::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer20RecipientLineFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::RecipientLineFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RecipientLineFactory, qt_meta_tag_ZN15MessageComposer20RecipientLineFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::RecipientLineFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<KPIM::MultiplyingLineFactory::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer20RecipientLineFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer20RecipientLineFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer20RecipientLineFactoryE_t>.metaTypes,
    nullptr
} };

void MessageComposer::RecipientLineFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RecipientLineFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *MessageComposer::RecipientLineFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::RecipientLineFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer20RecipientLineFactoryE_t>.strings))
        return static_cast<void*>(this);
    return KPIM::MultiplyingLineFactory::qt_metacast(_clname);
}

int MessageComposer::RecipientLineFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPIM::MultiplyingLineFactory::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15MessageComposer16RecipientsEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::RecipientsEditor::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer16RecipientsEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::RecipientsEditor",
        "focusInRecipientLineEdit",
        "",
        "slotPickedRecipient",
        "MessageComposer::Recipient",
        "bool&",
        "tooManyAddress",
        "slotLineAdded",
        "KPIM::MultiplyingLine*",
        "slotLineDeleted",
        "pos",
        "slotCalculateTotal",
        "addRecipient",
        "MessageComposer::RecipientLineNG*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'focusInRecipientLineEdit'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPickedRecipient'
        QtMocHelpers::SlotData<void(const MessageComposer::Recipient &, bool &)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 2 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotLineAdded'
        QtMocHelpers::SlotData<void(KPIM::MultiplyingLine *)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Slot 'slotLineDeleted'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'slotCalculateTotal'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addRecipient'
        QtMocHelpers::SlotData<void(MessageComposer::RecipientLineNG *, const QString &)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 2 }, { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RecipientsEditor, qt_meta_tag_ZN15MessageComposer16RecipientsEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::RecipientsEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<KPIM::MultiplyingLineEditor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer16RecipientsEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer16RecipientsEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer16RecipientsEditorE_t>.metaTypes,
    nullptr
} };

void MessageComposer::RecipientsEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RecipientsEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->focusInRecipientLineEdit(); break;
        case 1: _t->slotPickedRecipient((*reinterpret_cast<std::add_pointer_t<MessageComposer::Recipient>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[2]))); break;
        case 2: _t->slotLineAdded((*reinterpret_cast<std::add_pointer_t<KPIM::MultiplyingLine*>>(_a[1]))); break;
        case 3: _t->slotLineDeleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->slotCalculateTotal(); break;
        case 5: _t->addRecipient((*reinterpret_cast<std::add_pointer_t<MessageComposer::RecipientLineNG*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPIM::MultiplyingLine* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageComposer::RecipientLineNG* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RecipientsEditor::*)()>(_a, &RecipientsEditor::focusInRecipientLineEdit, 0))
            return;
    }
}

const QMetaObject *MessageComposer::RecipientsEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::RecipientsEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer16RecipientsEditorE_t>.strings))
        return static_cast<void*>(this);
    return KPIM::MultiplyingLineEditor::qt_metacast(_clname);
}

int MessageComposer::RecipientsEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPIM::MultiplyingLineEditor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void MessageComposer::RecipientsEditor::focusInRecipientLineEdit()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
