/********************************************************************************
** Form generated from reading UI file 'settings.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGS_H
#define UI_SETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_Settings
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *verticalSpacer_2;
    QLabel *label_3;
    QComboBox *overrideCharacterEncoding;
    QSpinBox *kcfg_MinimumFontSize;
    QVBoxLayout *verticalLayout;
    QCheckBox *kcfg_AccessKeyEnabled;
    QCheckBox *kcfg_ShrinkQuotes;
    QCheckBox *kcfg_ShowExpandQuotesMark;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QLabel *collapseQuoteLevelLabel;
    QSpinBox *kcfg_CollapseQuoteLevelSpin;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer;
    QLabel *label_4;

    void setupUi(QWidget *Settings)
    {
        if (Settings->objectName().isEmpty())
            Settings->setObjectName("Settings");
        Settings->resize(482, 456);
        gridLayout = new QGridLayout(Settings);
        gridLayout->setObjectName("gridLayout");
        verticalSpacer_2 = new QSpacerItem(20, 24, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        gridLayout->addItem(verticalSpacer_2, 1, 0, 1, 1);

        label_3 = new QLabel(Settings);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label_3, 2, 0, 1, 2);

        overrideCharacterEncoding = new QComboBox(Settings);
        overrideCharacterEncoding->setObjectName("overrideCharacterEncoding");
        sizePolicy.setHeightForWidth(overrideCharacterEncoding->sizePolicy().hasHeightForWidth());
        overrideCharacterEncoding->setSizePolicy(sizePolicy);

        gridLayout->addWidget(overrideCharacterEncoding, 2, 2, 1, 1);

        kcfg_MinimumFontSize = new QSpinBox(Settings);
        kcfg_MinimumFontSize->setObjectName("kcfg_MinimumFontSize");
        kcfg_MinimumFontSize->setMinimum(2);
        kcfg_MinimumFontSize->setMaximum(30);

        gridLayout->addWidget(kcfg_MinimumFontSize, 3, 2, 1, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        kcfg_AccessKeyEnabled = new QCheckBox(Settings);
        kcfg_AccessKeyEnabled->setObjectName("kcfg_AccessKeyEnabled");

        verticalLayout->addWidget(kcfg_AccessKeyEnabled);

        kcfg_ShrinkQuotes = new QCheckBox(Settings);
        kcfg_ShrinkQuotes->setObjectName("kcfg_ShrinkQuotes");

        verticalLayout->addWidget(kcfg_ShrinkQuotes);

        kcfg_ShowExpandQuotesMark = new QCheckBox(Settings);
        kcfg_ShowExpandQuotesMark->setObjectName("kcfg_ShowExpandQuotesMark");

        verticalLayout->addWidget(kcfg_ShowExpandQuotesMark);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer = new QSpacerItem(24, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        collapseQuoteLevelLabel = new QLabel(Settings);
        collapseQuoteLevelLabel->setObjectName("collapseQuoteLevelLabel");
        collapseQuoteLevelLabel->setEnabled(false);

        horizontalLayout_2->addWidget(collapseQuoteLevelLabel);

        kcfg_CollapseQuoteLevelSpin = new QSpinBox(Settings);
        kcfg_CollapseQuoteLevelSpin->setObjectName("kcfg_CollapseQuoteLevelSpin");
        kcfg_CollapseQuoteLevelSpin->setEnabled(false);
        kcfg_CollapseQuoteLevelSpin->setMinimum(1);
        kcfg_CollapseQuoteLevelSpin->setMaximum(3);
        kcfg_CollapseQuoteLevelSpin->setValue(3);

        horizontalLayout_2->addWidget(kcfg_CollapseQuoteLevelSpin);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout_2);


        gridLayout->addLayout(verticalLayout, 0, 0, 1, 3);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_3, 3, 3, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 9, 0, 1, 3);

        label_4 = new QLabel(Settings);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 3, 0, 1, 1);

#if QT_CONFIG(shortcut)
        label_3->setBuddy(overrideCharacterEncoding);
        collapseQuoteLevelLabel->setBuddy(kcfg_CollapseQuoteLevelSpin);
        label_4->setBuddy(kcfg_MinimumFontSize);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(kcfg_AccessKeyEnabled, kcfg_ShrinkQuotes);
        QWidget::setTabOrder(kcfg_ShrinkQuotes, kcfg_ShowExpandQuotesMark);
        QWidget::setTabOrder(kcfg_ShowExpandQuotesMark, kcfg_CollapseQuoteLevelSpin);
        QWidget::setTabOrder(kcfg_CollapseQuoteLevelSpin, overrideCharacterEncoding);
        QWidget::setTabOrder(overrideCharacterEncoding, kcfg_MinimumFontSize);

        retranslateUi(Settings);
        QObject::connect(kcfg_ShowExpandQuotesMark, &QCheckBox::toggled, collapseQuoteLevelLabel, &QLabel::setEnabled);
        QObject::connect(kcfg_ShowExpandQuotesMark, &QCheckBox::toggled, kcfg_CollapseQuoteLevelSpin, &QSpinBox::setEnabled);

        QMetaObject::connectSlotsByName(Settings);
    } // setupUi

    void retranslateUi(QWidget *Settings)
    {
        Settings->setWindowTitle(tr2i18n("Viewer settings", nullptr));
        label_3->setText(tr2i18n("O&verride character encoding:", nullptr));
        kcfg_AccessKeyEnabled->setText(tr2i18n("Enable access key", nullptr));
        kcfg_ShrinkQuotes->setText(tr2i18n("Reduce font size for &quoted text", nullptr));
        kcfg_ShowExpandQuotesMark->setText(tr2i18n("Show &expand/collapse quote marks", nullptr));
        collapseQuoteLevelLabel->setText(tr2i18n("Automatic collapse &level:", nullptr));
        label_4->setText(tr2i18n("&Minimum font size:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Settings: public Ui_Settings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SETTINGS_H

