/****************************************************************************
** Meta object code from reading C++ file 'widgetbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagelist/src/core/widgetbase.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'widgetbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11MessageList4Core6WidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageList::Core::Widget::qt_create_metaobjectdata<qt_meta_tag_ZN11MessageList4Core6WidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageList::Core::Widget",
        "statusMessage",
        "",
        "message",
        "forceLostFocus",
        "unlockTabRequested",
        "populateStatusFilterCombo",
        "changeQuicksearchVisibility",
        "setLockTab",
        "lock",
        "isLocked",
        "aggregationsChanged",
        "themesChanged",
        "themeMenuAboutToShow",
        "aggregationMenuAboutToShow",
        "themeSelected",
        "configureThemes",
        "setPrivateSortOrderForStorage",
        "aggregationSelected",
        "statusSelected",
        "index",
        "searchEditTextEdited",
        "searchTimerFired",
        "searchEditClearButtonClicked",
        "sortOrderMenuAboutToShow",
        "messageSortingSelected",
        "QAction*",
        "action",
        "messageSortDirectionSelected",
        "groupSortingSelected",
        "groupSortDirectionSelected",
        "resetFilter",
        "slotViewHeaderSectionClicked",
        "logicalIndex",
        "slotStatusButtonsClicked",
        "slotSaveFilter"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'statusMessage'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'forceLostFocus'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'unlockTabRequested'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'populateStatusFilterCombo'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'changeQuicksearchVisibility'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setLockTab'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'isLocked'
        QtMocHelpers::SlotData<bool() const>(10, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'aggregationsChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'themesChanged'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'themeMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'aggregationMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'themeSelected'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'configureThemes'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setPrivateSortOrderForStorage'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'aggregationSelected'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'statusSelected'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'searchEditTextEdited'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'searchTimerFired'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'searchEditClearButtonClicked'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'sortOrderMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'messageSortingSelected'
        QtMocHelpers::SlotData<void(QAction *)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'messageSortDirectionSelected'
        QtMocHelpers::SlotData<void(QAction *)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'groupSortingSelected'
        QtMocHelpers::SlotData<void(QAction *)>(29, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'groupSortDirectionSelected'
        QtMocHelpers::SlotData<void(QAction *)>(30, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'resetFilter'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotViewHeaderSectionClicked'
        QtMocHelpers::SlotData<void(int)>(32, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
        // Slot 'slotStatusButtonsClicked'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSaveFilter'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Widget, qt_meta_tag_ZN11MessageList4Core6WidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageList::Core::Widget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4Core6WidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4Core6WidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11MessageList4Core6WidgetE_t>.metaTypes,
    nullptr
} };

void MessageList::Core::Widget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Widget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->statusMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->forceLostFocus(); break;
        case 2: _t->unlockTabRequested(); break;
        case 3: _t->populateStatusFilterCombo(); break;
        case 4: _t->changeQuicksearchVisibility((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->setLockTab((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: { bool _r = _t->isLocked();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->aggregationsChanged(); break;
        case 8: _t->themesChanged(); break;
        case 9: _t->themeMenuAboutToShow(); break;
        case 10: _t->aggregationMenuAboutToShow(); break;
        case 11: _t->themeSelected((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->configureThemes(); break;
        case 13: _t->setPrivateSortOrderForStorage(); break;
        case 14: _t->aggregationSelected((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->statusSelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->searchEditTextEdited(); break;
        case 17: _t->searchTimerFired(); break;
        case 18: _t->searchEditClearButtonClicked(); break;
        case 19: _t->sortOrderMenuAboutToShow(); break;
        case 20: _t->messageSortingSelected((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 21: _t->messageSortDirectionSelected((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 22: _t->groupSortingSelected((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 23: _t->groupSortDirectionSelected((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 24: _t->resetFilter(); break;
        case 25: _t->slotViewHeaderSectionClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->slotStatusButtonsClicked(); break;
        case 27: _t->slotSaveFilter(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Widget::*)(const QString & )>(_a, &Widget::statusMessage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Widget::*)()>(_a, &Widget::forceLostFocus, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Widget::*)()>(_a, &Widget::unlockTabRequested, 2))
            return;
    }
}

const QMetaObject *MessageList::Core::Widget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageList::Core::Widget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4Core6WidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int MessageList::Core::Widget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void MessageList::Core::Widget::statusMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MessageList::Core::Widget::forceLostFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MessageList::Core::Widget::unlockTabRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
