/********************************************************************************
** Form generated from reading UI file 'attachmentpropertiesdialog_readonly.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ATTACHMENTPROPERTIESDIALOG_READONLY_H
#define UI_ATTACHMENTPROPERTIESDIALOG_READONLY_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_AttachmentPropertiesDialogReadOnly
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *mimeType;
    QLabel *description;
    QLabel *name;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QLabel *mimeIcon;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_2;
    QLabel *size;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout_4;
    QLabel *descriptionLabel;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_5;
    QLabel *encoding;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *AttachmentPropertiesDialogReadOnly)
    {
        if (AttachmentPropertiesDialogReadOnly->objectName().isEmpty())
            AttachmentPropertiesDialogReadOnly->setObjectName("AttachmentPropertiesDialogReadOnly");
        AttachmentPropertiesDialogReadOnly->resize(371, 252);
        verticalLayout = new QVBoxLayout(AttachmentPropertiesDialogReadOnly);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        mimeType = new QLabel(AttachmentPropertiesDialogReadOnly);
        mimeType->setObjectName("mimeType");
        mimeType->setText(QString::fromUtf8("TextLabel"));

        gridLayout->addWidget(mimeType, 0, 1, 1, 1);

        description = new QLabel(AttachmentPropertiesDialogReadOnly);
        description->setObjectName("description");
        description->setText(QString::fromUtf8("TextLabel"));

        gridLayout->addWidget(description, 3, 1, 1, 2);

        name = new QLabel(AttachmentPropertiesDialogReadOnly);
        name->setObjectName("name");
        name->setText(QString::fromUtf8("TextLabel"));

        gridLayout->addWidget(name, 2, 1, 1, 2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(AttachmentPropertiesDialogReadOnly);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(label);


        gridLayout->addLayout(horizontalLayout, 0, 0, 1, 1);

        mimeIcon = new QLabel(AttachmentPropertiesDialogReadOnly);
        mimeIcon->setObjectName("mimeIcon");
        mimeIcon->setText(QString::fromUtf8("pretty icon"));

        gridLayout->addWidget(mimeIcon, 0, 2, 2, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_2 = new QLabel(AttachmentPropertiesDialogReadOnly);
        label_2->setObjectName("label_2");
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(label_2);


        gridLayout->addLayout(horizontalLayout_2, 1, 0, 1, 1);

        size = new QLabel(AttachmentPropertiesDialogReadOnly);
        size->setObjectName("size");
        size->setText(QString::fromUtf8("size goes here"));

        gridLayout->addWidget(size, 1, 1, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_3 = new QLabel(AttachmentPropertiesDialogReadOnly);
        label_3->setObjectName("label_3");
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_3->addWidget(label_3);


        gridLayout->addLayout(horizontalLayout_3, 2, 0, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        descriptionLabel = new QLabel(AttachmentPropertiesDialogReadOnly);
        descriptionLabel->setObjectName("descriptionLabel");
        sizePolicy.setHeightForWidth(descriptionLabel->sizePolicy().hasHeightForWidth());
        descriptionLabel->setSizePolicy(sizePolicy);
        descriptionLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_4->addWidget(descriptionLabel);


        gridLayout->addLayout(horizontalLayout_4, 3, 0, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_5 = new QLabel(AttachmentPropertiesDialogReadOnly);
        label_5->setObjectName("label_5");
        sizePolicy.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy);
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_5->addWidget(label_5);


        gridLayout->addLayout(horizontalLayout_5, 4, 0, 1, 1);

        encoding = new QLabel(AttachmentPropertiesDialogReadOnly);
        encoding->setObjectName("encoding");
        encoding->setText(QString::fromUtf8("TextLabel"));

        gridLayout->addWidget(encoding, 4, 1, 1, 2);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 31, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(AttachmentPropertiesDialogReadOnly);

        QMetaObject::connectSlotsByName(AttachmentPropertiesDialogReadOnly);
    } // setupUi

    void retranslateUi(QWidget *AttachmentPropertiesDialogReadOnly)
    {
        label->setText(tr2i18n("Type:", "MIME type of this attachment"));
        label_2->setText(tr2i18n("Size:", "size of the attachment"));
        label_3->setText(tr2i18n("Name:", "file name of the attachment"));
        descriptionLabel->setText(tr2i18n("Description:", "description of the attachment"));
        label_5->setText(tr2i18n("Encoding:", "encoding of the attachment"));
        (void)AttachmentPropertiesDialogReadOnly;
    } // retranslateUi

};

namespace Ui {
    class AttachmentPropertiesDialogReadOnly: public Ui_AttachmentPropertiesDialogReadOnly {};
} // namespace Ui

QT_END_NAMESPACE

#endif // ATTACHMENTPROPERTIESDIALOG_READONLY_H

