/****************************************************************************
** Meta object code from reading C++ file 'attachmentcontrollerbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagecomposer/src/attachment/attachmentcontrollerbase.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'attachmentcontrollerbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15MessageComposer24AttachmentControllerBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::AttachmentControllerBase::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer24AttachmentControllerBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::AttachmentControllerBase",
        "actionsCreated",
        "",
        "refreshSelection",
        "showAttachment",
        "std::shared_ptr<KMime::Content>",
        "content",
        "charset",
        "selectedAllAttachment",
        "addOwnVcard",
        "fileAttached",
        "setEncryptEnabled",
        "enabled",
        "setSignEnabled",
        "compressAttachment",
        "MessageCore::AttachmentPart::Ptr",
        "part",
        "compress",
        "showContextMenu",
        "openAttachment",
        "viewAttachment",
        "editAttachment",
        "MessageComposer::EditorWatcher::OpenWithOption",
        "option",
        "editAttachmentWith",
        "saveAttachmentAs",
        "attachmentProperties",
        "showAddAttachmentFileDialog",
        "showAddAttachmentCompressedDirectoryDialog",
        "addAttachment",
        "QUrl",
        "url",
        "addAttachmentUrlSync",
        "addAttachments",
        "QList<QUrl>",
        "urls",
        "showAttachPublicKeyDialog",
        "showAttachVcard",
        "showAttachClipBoard",
        "attachMyPublicKey"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'actionsCreated'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'refreshSelection'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showAttachment'
        QtMocHelpers::SignalData<void(const std::shared_ptr<KMime::Content> &, const QByteArray &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QByteArray, 7 },
        }}),
        // Signal 'selectedAllAttachment'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addOwnVcard'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'fileAttached'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setEncryptEnabled'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'setSignEnabled'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'compressAttachment'
        QtMocHelpers::SlotData<void(const MessageCore::AttachmentPart::Ptr &, bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { QMetaType::Bool, 17 },
        }}),
        // Slot 'showContextMenu'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openAttachment'
        QtMocHelpers::SlotData<void(const MessageCore::AttachmentPart::Ptr &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'viewAttachment'
        QtMocHelpers::SlotData<void(const MessageCore::AttachmentPart::Ptr &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'editAttachment'
        QtMocHelpers::SlotData<void(MessageCore::AttachmentPart::Ptr, MessageComposer::EditorWatcher::OpenWithOption)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'editAttachment'
        QtMocHelpers::SlotData<void(MessageCore::AttachmentPart::Ptr)>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'editAttachmentWith'
        QtMocHelpers::SlotData<void(const MessageCore::AttachmentPart::Ptr &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'saveAttachmentAs'
        QtMocHelpers::SlotData<void(const MessageCore::AttachmentPart::Ptr &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'attachmentProperties'
        QtMocHelpers::SlotData<void(const MessageCore::AttachmentPart::Ptr &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'showAddAttachmentFileDialog'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showAddAttachmentCompressedDirectoryDialog'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addAttachment'
        QtMocHelpers::SlotData<void(const MessageCore::AttachmentPart::Ptr &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'addAttachment'
        QtMocHelpers::SlotData<void(const QUrl &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'addAttachmentUrlSync'
        QtMocHelpers::SlotData<void(const QUrl &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'addAttachments'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Slot 'showAttachPublicKeyDialog'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showAttachVcard'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showAttachClipBoard'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'attachMyPublicKey'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AttachmentControllerBase, qt_meta_tag_ZN15MessageComposer24AttachmentControllerBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::AttachmentControllerBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer24AttachmentControllerBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer24AttachmentControllerBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer24AttachmentControllerBaseE_t>.metaTypes,
    nullptr
} };

void MessageComposer::AttachmentControllerBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AttachmentControllerBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->actionsCreated(); break;
        case 1: _t->refreshSelection(); break;
        case 2: _t->showAttachment((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<KMime::Content>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 3: _t->selectedAllAttachment(); break;
        case 4: _t->addOwnVcard((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->fileAttached(); break;
        case 6: _t->setEncryptEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setSignEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->compressAttachment((*reinterpret_cast<std::add_pointer_t<MessageCore::AttachmentPart::Ptr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->showContextMenu(); break;
        case 10: _t->openAttachment((*reinterpret_cast<std::add_pointer_t<MessageCore::AttachmentPart::Ptr>>(_a[1]))); break;
        case 11: _t->viewAttachment((*reinterpret_cast<std::add_pointer_t<MessageCore::AttachmentPart::Ptr>>(_a[1]))); break;
        case 12: _t->editAttachment((*reinterpret_cast<std::add_pointer_t<MessageCore::AttachmentPart::Ptr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MessageComposer::EditorWatcher::OpenWithOption>>(_a[2]))); break;
        case 13: _t->editAttachment((*reinterpret_cast<std::add_pointer_t<MessageCore::AttachmentPart::Ptr>>(_a[1]))); break;
        case 14: _t->editAttachmentWith((*reinterpret_cast<std::add_pointer_t<MessageCore::AttachmentPart::Ptr>>(_a[1]))); break;
        case 15: _t->saveAttachmentAs((*reinterpret_cast<std::add_pointer_t<MessageCore::AttachmentPart::Ptr>>(_a[1]))); break;
        case 16: _t->attachmentProperties((*reinterpret_cast<std::add_pointer_t<MessageCore::AttachmentPart::Ptr>>(_a[1]))); break;
        case 17: _t->showAddAttachmentFileDialog(); break;
        case 18: _t->showAddAttachmentCompressedDirectoryDialog(); break;
        case 19: _t->addAttachment((*reinterpret_cast<std::add_pointer_t<MessageCore::AttachmentPart::Ptr>>(_a[1]))); break;
        case 20: _t->addAttachment((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 21: _t->addAttachmentUrlSync((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 22: _t->addAttachments((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 23: _t->showAttachPublicKeyDialog(); break;
        case 24: _t->showAttachVcard(); break;
        case 25: _t->showAttachClipBoard(); break;
        case 26: _t->attachMyPublicKey(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageCore::AttachmentPart::Ptr >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageCore::AttachmentPart::Ptr >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageCore::AttachmentPart::Ptr >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageCore::AttachmentPart::Ptr >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageCore::AttachmentPart::Ptr >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageCore::AttachmentPart::Ptr >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageCore::AttachmentPart::Ptr >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageCore::AttachmentPart::Ptr >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageCore::AttachmentPart::Ptr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AttachmentControllerBase::*)()>(_a, &AttachmentControllerBase::actionsCreated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AttachmentControllerBase::*)()>(_a, &AttachmentControllerBase::refreshSelection, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AttachmentControllerBase::*)(const std::shared_ptr<KMime::Content> & , const QByteArray & )>(_a, &AttachmentControllerBase::showAttachment, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AttachmentControllerBase::*)()>(_a, &AttachmentControllerBase::selectedAllAttachment, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AttachmentControllerBase::*)(bool )>(_a, &AttachmentControllerBase::addOwnVcard, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AttachmentControllerBase::*)()>(_a, &AttachmentControllerBase::fileAttached, 5))
            return;
    }
}

const QMetaObject *MessageComposer::AttachmentControllerBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::AttachmentControllerBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer24AttachmentControllerBaseE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MessageComposer::AttachmentControllerBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void MessageComposer::AttachmentControllerBase::actionsCreated()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MessageComposer::AttachmentControllerBase::refreshSelection()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MessageComposer::AttachmentControllerBase::showAttachment(const std::shared_ptr<KMime::Content> & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void MessageComposer::AttachmentControllerBase::selectedAllAttachment()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MessageComposer::AttachmentControllerBase::addOwnVcard(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MessageComposer::AttachmentControllerBase::fileAttached()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
