// This file is generated by kconfig_compiler_kf6 from defaultgrantleeheaderstyleplugin.kcfg.
// All changes you do to this file will be lost.
#ifndef DEFAULTGRANTLEEHEADERSTYLEPLUGINSETTINGS_H
#define DEFAULTGRANTLEEHEADERSTYLEPLUGINSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class DefaultGrantleeHeaderStylePluginSettings : public KConfigSkeleton
{
  public:

    static DefaultGrantleeHeaderStylePluginSettings *self();
    ~DefaultGrantleeHeaderStylePluginSettings() override;

    /**
      Set ThemeName
    */
    static
    void setThemeName( const QString & v )
    {
      if (!self()->isThemeNameImmutable())
        self()->mThemeName = v;
    }

    /**
      Get ThemeName
    */
    static
    QString themeName()
    {
      return self()->mThemeName;
    }

    /**
      Is ThemeName Immutable
    */
    static
    bool isThemeNameImmutable()
    {
      return self()->isImmutable( QStringLiteral( "ThemeName" ) );
    }

    /**
      Get Item object corresponding to ThemeName()
    */
    ItemString *themeNameItem()
    {
      return mThemeNameItem;
    }

  protected:
    DefaultGrantleeHeaderStylePluginSettings();
    friend class DefaultGrantleeHeaderStylePluginSettingsHelper;


    // GrantleeDefaultTheme
    QString mThemeName;

  private:
    ItemString *mThemeNameItem;
};

#endif

