/*  -*- mode: C++; c-file-style: "gnu" -*-
    bodypartformatter.cpp

    This file is part of KMail's plugin interface.
    SPDX-FileCopyrightText: 2016 Sandro Knauß <sknauss@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "bodypartformatter.h"

using namespace MimeTreeParser::Interface;

BodyPartFormatter::~BodyPartFormatter() = default;
BodyPartFormatterPlugin::~BodyPartFormatterPlugin() = default;
