/****************************************************************************
** Meta object code from reading C++ file 'viewer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messageviewer/src/viewer/viewer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'viewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13MessageViewer6ViewerE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageViewer::Viewer::qt_create_metaobjectdata<qt_meta_tag_ZN13MessageViewer6ViewerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageViewer::Viewer",
        "moveMessageToTrash",
        "",
        "pageIsScrolledToBottom",
        "showStatusBarMessage",
        "message",
        "popupMenu",
        "Akonadi::Item",
        "msg",
        "QUrl",
        "url",
        "imageUrl",
        "QPoint",
        "mousePos",
        "displayPopupMenu",
        "WebEngineViewer::WebHitTestResult",
        "result",
        "urlClicked",
        "requestConfigSync",
        "showReader",
        "KMime::Content*",
        "aMsgPart",
        "aHTML",
        "encoding",
        "showMessage",
        "std::shared_ptr<KMime::Message>",
        "replyMessageTo",
        "replyToAll",
        "deleteMessage",
        "itemRemoved",
        "makeResourceOnline",
        "MessageViewer::Viewer::ResourceOnlineMode",
        "mode",
        "printingFinished",
        "zoomChanged",
        "zoomFactor",
        "showNextMessage",
        "showPreviousMessage",
        "sendResponse",
        "MessageViewer::MDNWarningWidget::ResponseType",
        "type",
        "KMime::MDN::SendingMode",
        "sendingMode",
        "slotScrollUp",
        "slotScrollDown",
        "slotScrollPrior",
        "slotScrollNext",
        "slotJumpDown",
        "slotFind",
        "slotSaveMessage",
        "slotAttachmentSaveAs",
        "slotAttachmentSaveAll",
        "slotShowMessageSource",
        "slotZoomIn",
        "slotZoomOut",
        "slotZoomReset",
        "slotChangeDisplayMail",
        "MessageViewer::Viewer::DisplayFormatMessage",
        "DisplayFormatMessage",
        "UseGlobalSetting",
        "Text",
        "Html",
        "Unknown",
        "ICal",
        "AttachmentAction",
        "Open",
        "OpenWith",
        "View",
        "Save",
        "Properties",
        "Delete",
        "Copy",
        "ScrollTo",
        "ReplyMessageToAuthor",
        "ReplyMessageToAll",
        "ResourceOnlineMode",
        "AllResources",
        "SelectedResource"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'moveMessageToTrash'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pageIsScrolledToBottom'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'showStatusBarMessage'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'popupMenu'
        QtMocHelpers::SignalData<void(const Akonadi::Item &, const QUrl &, const QUrl &, const QPoint &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Signal 'displayPopupMenu'
        QtMocHelpers::SignalData<void(const Akonadi::Item &, const WebEngineViewer::WebHitTestResult &, const QPoint &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 15, 16 }, { 0x80000000 | 12, 13 },
        }}),
        // Signal 'urlClicked'
        QtMocHelpers::SignalData<void(const Akonadi::Item &, const QUrl &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 }, { 0x80000000 | 9, 2 },
        }}),
        // Signal 'requestConfigSync'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showReader'
        QtMocHelpers::SignalData<void(KMime::Content *, bool, const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::Bool, 22 }, { QMetaType::QString, 23 },
        }}),
        // Signal 'showMessage'
        QtMocHelpers::SignalData<void(const std::shared_ptr<KMime::Message> &, const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 5 }, { QMetaType::QString, 23 },
        }}),
        // Signal 'replyMessageTo'
        QtMocHelpers::SignalData<void(const std::shared_ptr<KMime::Message> &, bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 5 }, { QMetaType::Bool, 27 },
        }}),
        // Signal 'deleteMessage'
        QtMocHelpers::SignalData<void(const Akonadi::Item &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'itemRemoved'
        QtMocHelpers::SignalData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'makeResourceOnline'
        QtMocHelpers::SignalData<void(MessageViewer::Viewer::ResourceOnlineMode)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Signal 'printingFinished'
        QtMocHelpers::SignalData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'zoomChanged'
        QtMocHelpers::SignalData<void(qreal)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 35 },
        }}),
        // Signal 'showNextMessage'
        QtMocHelpers::SignalData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showPreviousMessage'
        QtMocHelpers::SignalData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sendResponse'
        QtMocHelpers::SignalData<void(MessageViewer::MDNWarningWidget::ResponseType, KMime::MDN::SendingMode)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 40 }, { 0x80000000 | 41, 42 },
        }}),
        // Slot 'slotScrollUp'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotScrollDown'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotScrollPrior'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotScrollNext'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotJumpDown'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFind'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSaveMessage'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAttachmentSaveAs'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAttachmentSaveAll'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowMessageSource'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotZoomIn'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotZoomOut'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotZoomReset'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotChangeDisplayMail'
        QtMocHelpers::SlotData<void(MessageViewer::Viewer::DisplayFormatMessage, bool)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 2 }, { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DisplayFormatMessage'
        QtMocHelpers::EnumData<enum DisplayFormatMessage>(58, 58, QMC::EnumFlags{}).add({
            {   59, DisplayFormatMessage::UseGlobalSetting },
            {   60, DisplayFormatMessage::Text },
            {   61, DisplayFormatMessage::Html },
            {   62, DisplayFormatMessage::Unknown },
            {   63, DisplayFormatMessage::ICal },
        }),
        // enum 'AttachmentAction'
        QtMocHelpers::EnumData<enum AttachmentAction>(64, 64, QMC::EnumFlags{}).add({
            {   65, AttachmentAction::Open },
            {   66, AttachmentAction::OpenWith },
            {   67, AttachmentAction::View },
            {   68, AttachmentAction::Save },
            {   69, AttachmentAction::Properties },
            {   70, AttachmentAction::Delete },
            {   71, AttachmentAction::Copy },
            {   72, AttachmentAction::ScrollTo },
            {   73, AttachmentAction::ReplyMessageToAuthor },
            {   74, AttachmentAction::ReplyMessageToAll },
        }),
        // enum 'ResourceOnlineMode'
        QtMocHelpers::EnumData<enum ResourceOnlineMode>(75, 75, QMC::EnumFlags{}).add({
            {   76, ResourceOnlineMode::AllResources },
            {   77, ResourceOnlineMode::SelectedResource },
        }),
    };
    return QtMocHelpers::metaObjectData<Viewer, qt_meta_tag_ZN13MessageViewer6ViewerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageViewer::Viewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer6ViewerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer6ViewerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13MessageViewer6ViewerE_t>.metaTypes,
    nullptr
} };

void MessageViewer::Viewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Viewer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->moveMessageToTrash(); break;
        case 1: _t->pageIsScrolledToBottom((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->showStatusBarMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->popupMenu((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[4]))); break;
        case 4: _t->displayPopupMenu((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<WebEngineViewer::WebHitTestResult>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[3]))); break;
        case 5: _t->urlClicked((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 6: _t->requestConfigSync(); break;
        case 7: _t->showReader((*reinterpret_cast<std::add_pointer_t<KMime::Content*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 8: _t->showMessage((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<KMime::Message>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->replyMessageTo((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<KMime::Message>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 10: _t->deleteMessage((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1]))); break;
        case 11: _t->itemRemoved(); break;
        case 12: _t->makeResourceOnline((*reinterpret_cast<std::add_pointer_t<MessageViewer::Viewer::ResourceOnlineMode>>(_a[1]))); break;
        case 13: _t->printingFinished(); break;
        case 14: _t->zoomChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 15: _t->showNextMessage(); break;
        case 16: _t->showPreviousMessage(); break;
        case 17: _t->sendResponse((*reinterpret_cast<std::add_pointer_t<MessageViewer::MDNWarningWidget::ResponseType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KMime::MDN::SendingMode>>(_a[2]))); break;
        case 18: _t->slotScrollUp(); break;
        case 19: _t->slotScrollDown(); break;
        case 20: _t->slotScrollPrior(); break;
        case 21: _t->slotScrollNext(); break;
        case 22: _t->slotJumpDown(); break;
        case 23: _t->slotFind(); break;
        case 24: _t->slotSaveMessage(); break;
        case 25: _t->slotAttachmentSaveAs(); break;
        case 26: _t->slotAttachmentSaveAll(); break;
        case 27: _t->slotShowMessageSource(); break;
        case 28: _t->slotZoomIn(); break;
        case 29: _t->slotZoomOut(); break;
        case 30: _t->slotZoomReset(); break;
        case 31: _t->slotChangeDisplayMail((*reinterpret_cast<std::add_pointer_t<MessageViewer::Viewer::DisplayFormatMessage>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KMime::Content* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< std::shared_ptr<KMime::Message> >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< std::shared_ptr<KMime::Message> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)()>(_a, &Viewer::moveMessageToTrash, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(bool )>(_a, &Viewer::pageIsScrolledToBottom, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(const QString & )>(_a, &Viewer::showStatusBarMessage, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(const Akonadi::Item & , const QUrl & , const QUrl & , const QPoint & )>(_a, &Viewer::popupMenu, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(const Akonadi::Item & , const WebEngineViewer::WebHitTestResult & , const QPoint & )>(_a, &Viewer::displayPopupMenu, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(const Akonadi::Item & , const QUrl & )>(_a, &Viewer::urlClicked, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)()>(_a, &Viewer::requestConfigSync, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(KMime::Content * , bool , const QString & )>(_a, &Viewer::showReader, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(const std::shared_ptr<KMime::Message> & , const QString & )>(_a, &Viewer::showMessage, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(const std::shared_ptr<KMime::Message> & , bool )>(_a, &Viewer::replyMessageTo, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(const Akonadi::Item & )>(_a, &Viewer::deleteMessage, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)()>(_a, &Viewer::itemRemoved, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(MessageViewer::Viewer::ResourceOnlineMode )>(_a, &Viewer::makeResourceOnline, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)()>(_a, &Viewer::printingFinished, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(qreal )>(_a, &Viewer::zoomChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)()>(_a, &Viewer::showNextMessage, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)()>(_a, &Viewer::showPreviousMessage, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Viewer::*)(MessageViewer::MDNWarningWidget::ResponseType , KMime::MDN::SendingMode )>(_a, &Viewer::sendResponse, 17))
            return;
    }
}

const QMetaObject *MessageViewer::Viewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageViewer::Viewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer6ViewerE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int MessageViewer::Viewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    return _id;
}

// SIGNAL 0
void MessageViewer::Viewer::moveMessageToTrash()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MessageViewer::Viewer::pageIsScrolledToBottom(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MessageViewer::Viewer::showStatusBarMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MessageViewer::Viewer::popupMenu(const Akonadi::Item & _t1, const QUrl & _t2, const QUrl & _t3, const QPoint & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 4
void MessageViewer::Viewer::displayPopupMenu(const Akonadi::Item & _t1, const WebEngineViewer::WebHitTestResult & _t2, const QPoint & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}

// SIGNAL 5
void MessageViewer::Viewer::urlClicked(const Akonadi::Item & _t1, const QUrl & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void MessageViewer::Viewer::requestConfigSync()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void MessageViewer::Viewer::showReader(KMime::Content * _t1, bool _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3);
}

// SIGNAL 8
void MessageViewer::Viewer::showMessage(const std::shared_ptr<KMime::Message> & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void MessageViewer::Viewer::replyMessageTo(const std::shared_ptr<KMime::Message> & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void MessageViewer::Viewer::deleteMessage(const Akonadi::Item & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void MessageViewer::Viewer::itemRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void MessageViewer::Viewer::makeResourceOnline(MessageViewer::Viewer::ResourceOnlineMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void MessageViewer::Viewer::printingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void MessageViewer::Viewer::zoomChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void MessageViewer::Viewer::showNextMessage()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void MessageViewer::Viewer::showPreviousMessage()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void MessageViewer::Viewer::sendResponse(MessageViewer::MDNWarningWidget::ResponseType _t1, KMime::MDN::SendingMode _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1, _t2);
}
QT_WARNING_POP
