// This file is generated by kconfig_compiler_kf6 from templatesconfiguration_kfg.kcfg.
// All changes you do to this file will be lost.

#include "templatesconfiguration_kfg.h"

using namespace TemplateParser;

Templates::Templates( const QString & folder )
  : KConfigSkeleton( QStringLiteral( "templatesconfigurationrc" ) )
  , mParamfolder(folder)
{
  setCurrentGroup( QStringLiteral( "Templates #%1" ).arg( mParamfolder ) );

  mUseCustomTemplatesItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "UseCustomTemplates" ), mUseCustomTemplates, false );
  addItem( mUseCustomTemplatesItem, QStringLiteral( "UseCustomTemplates" ) );
  mTemplateNewMessageItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "TemplateNewMessage" ), mTemplateNewMessage );
  addItem( mTemplateNewMessageItem, QStringLiteral( "TemplateNewMessage" ) );
  mTemplateReplyItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "TemplateReply" ), mTemplateReply );
  addItem( mTemplateReplyItem, QStringLiteral( "TemplateReply" ) );
  mTemplateReplyAllItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "TemplateReplyAll" ), mTemplateReplyAll );
  addItem( mTemplateReplyAllItem, QStringLiteral( "TemplateReplyAll" ) );
  mTemplateForwardItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "TemplateForward" ), mTemplateForward );
  addItem( mTemplateForwardItem, QStringLiteral( "TemplateForward" ) );
  mQuoteStringItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "QuoteString" ), mQuoteString );
  addItem( mQuoteStringItem, QStringLiteral( "QuoteString" ) );
}

Templates::~Templates()
{
}

