// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "webengineviewer_block_tracking_url_interceptor_debug.h"


Q_LOGGING_CATEGORY(WEBENGINEVIEWER_BLOCK_TRACKING_URL_LOG, "org.kde.pim.webengineviewer_block_tracking_url", QtInfoMsg)

