// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "messageviewer_dkimcheckerdebug.h"


Q_LOGGING_CATEGORY(MESSAGEVIEWER_DKIMCHECKER_LOG, "org.kde.pim.messageviewer_dkimchecker", QtInfoMsg)

