// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "messageviewer_viewer_format_debug.h"


Q_LOGGING_CATEGORY(MESSAGEVIEWER_VIEWER_FORMAT_LOG, "org.kde.pim.messageviewer_viewer_format", QtInfoMsg)

