// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "messagelist_pane_debug.h"


Q_LOGGING_CATEGORY(MESSAGELIST_PANE_LOG, "org.kde.pim.messagelist_pane", QtInfoMsg)

