/****************************************************************************
** Meta object code from reading C++ file 'htmlstatusbar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messageviewer/src/widgets/htmlstatusbar.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'htmlstatusbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13MessageViewer13HtmlStatusBarE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageViewer::HtmlStatusBar::qt_create_metaobjectdata<qt_meta_tag_ZN13MessageViewer13HtmlStatusBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageViewer::HtmlStatusBar",
        "clicked",
        "",
        "setHtmlMode",
        "setNormalMode",
        "setMode",
        "MimeTreeParser::Util::HtmlMode",
        "m",
        "MessageViewer::HtmlStatusBar::UpdateMode",
        "mode",
        "UpdateMode",
        "NoUpdate",
        "Update"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setHtmlMode'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setNormalMode'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setMode'
        QtMocHelpers::SlotData<void(MimeTreeParser::Util::HtmlMode, MessageViewer::HtmlStatusBar::UpdateMode)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Slot 'setMode'
        QtMocHelpers::SlotData<void(MimeTreeParser::Util::HtmlMode)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'UpdateMode'
        QtMocHelpers::EnumData<enum UpdateMode>(10, 10, QMC::EnumFlags{}).add({
            {   11, UpdateMode::NoUpdate },
            {   12, UpdateMode::Update },
        }),
    };
    return QtMocHelpers::metaObjectData<HtmlStatusBar, qt_meta_tag_ZN13MessageViewer13HtmlStatusBarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageViewer::HtmlStatusBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QLabel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer13HtmlStatusBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer13HtmlStatusBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13MessageViewer13HtmlStatusBarE_t>.metaTypes,
    nullptr
} };

void MessageViewer::HtmlStatusBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HtmlStatusBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked(); break;
        case 1: _t->setHtmlMode(); break;
        case 2: _t->setNormalMode(); break;
        case 3: _t->setMode((*reinterpret_cast<std::add_pointer_t<MimeTreeParser::Util::HtmlMode>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MessageViewer::HtmlStatusBar::UpdateMode>>(_a[2]))); break;
        case 4: _t->setMode((*reinterpret_cast<std::add_pointer_t<MimeTreeParser::Util::HtmlMode>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (HtmlStatusBar::*)()>(_a, &HtmlStatusBar::clicked, 0))
            return;
    }
}

const QMetaObject *MessageViewer::HtmlStatusBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageViewer::HtmlStatusBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageViewer13HtmlStatusBarE_t>.strings))
        return static_cast<void*>(this);
    return QLabel::qt_metacast(_clname);
}

int MessageViewer::HtmlStatusBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLabel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void MessageViewer::HtmlStatusBar::clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
