/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.Akonadi.SendLaterAgent.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef SENDLATERINTERFACE_H
#define SENDLATERINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.freedesktop.Akonadi.SendLaterAgent
 */
class OrgFreedesktopAkonadiSendLaterAgentInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.Akonadi.SendLaterAgent"; }

public:
    OrgFreedesktopAkonadiSendLaterAgentInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgFreedesktopAkonadiSendLaterAgentInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> addItem(qlonglong datetime, bool recurrence, int recurrenceValue, int recurrenceUnit, qlonglong itemId, const QString &subject, const QString &to)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(datetime) << QVariant::fromValue(recurrence) << QVariant::fromValue(recurrenceValue) << QVariant::fromValue(recurrenceUnit) << QVariant::fromValue(itemId) << QVariant::fromValue(subject) << QVariant::fromValue(to);
        return asyncCallWithArgumentList(QStringLiteral("addItem"), argumentList);
    }

    inline QDBusPendingReply<bool> enabledAgent()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("enabledAgent"), argumentList);
    }

    inline QDBusPendingReply<QString> printDebugInfo()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("printDebugInfo"), argumentList);
    }

    inline QDBusPendingReply<> reload()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("reload"), argumentList);
    }

    inline QDBusPendingReply<> removeItem(qlonglong item)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(item);
        return asyncCallWithArgumentList(QStringLiteral("removeItem"), argumentList);
    }

    inline QDBusPendingReply<> setEnableAgent(bool in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setEnableAgent"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace freedesktop {
    namespace Akonadi {
      using SendLaterAgent = ::OrgFreedesktopAkonadiSendLaterAgentInterface;
    }
  }
}

#endif
