/****************************************************************************
** Meta object code from reading C++ file 'textpart.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagecomposer/src/part/textpart.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textpart.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15MessageComposer8TextPartE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::TextPart::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer8TextPartE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::TextPart",
        "wordWrappingChanged",
        "",
        "warnBadCharsetChanged",
        "cleanPlainTextChanged",
        "wrappedPlainTextChanged",
        "cleanHtmlChanged",
        "embeddedImagesChanged",
        "isWordWrappingEnabled",
        "warnBadCharset",
        "cleanPlainText",
        "wrappedPlainText",
        "cleanHtml",
        "isHtmlUsed",
        "hasEmbeddedImages",
        "embeddedImages",
        "KPIMTextEdit::ImageList"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'wordWrappingChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'warnBadCharsetChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cleanPlainTextChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'wrappedPlainTextChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cleanHtmlChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'embeddedImagesChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isWordWrappingEnabled'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'warnBadCharset'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'cleanPlainText'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'wrappedPlainText'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'cleanHtml'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'isHtmlUsed'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'hasEmbeddedImages'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags, 5),
        // property 'embeddedImages'
        QtMocHelpers::PropertyData<KPIMTextEdit::ImageList>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextPart, qt_meta_tag_ZN15MessageComposer8TextPartE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::TextPart::staticMetaObject = { {
    QMetaObject::SuperData::link<MessagePart::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer8TextPartE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer8TextPartE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer8TextPartE_t>.metaTypes,
    nullptr
} };

void MessageComposer::TextPart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextPart *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->wordWrappingChanged(); break;
        case 1: _t->warnBadCharsetChanged(); break;
        case 2: _t->cleanPlainTextChanged(); break;
        case 3: _t->wrappedPlainTextChanged(); break;
        case 4: _t->cleanHtmlChanged(); break;
        case 5: _t->embeddedImagesChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextPart::*)()>(_a, &TextPart::wordWrappingChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextPart::*)()>(_a, &TextPart::warnBadCharsetChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextPart::*)()>(_a, &TextPart::cleanPlainTextChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextPart::*)()>(_a, &TextPart::wrappedPlainTextChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextPart::*)()>(_a, &TextPart::cleanHtmlChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextPart::*)()>(_a, &TextPart::embeddedImagesChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isWordWrappingEnabled(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->warnBadCharset(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->cleanPlainText(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->wrappedPlainText(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->cleanHtml(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isHtmlUsed(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->hasEmbeddedImages(); break;
        case 7: *reinterpret_cast<KPIMTextEdit::ImageList*>(_v) = _t->embeddedImages(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWordWrappingEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setWarnBadCharset(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setCleanPlainText(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setWrappedPlainText(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setCleanHtml(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setEmbeddedImages(*reinterpret_cast<KPIMTextEdit::ImageList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *MessageComposer::TextPart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::TextPart::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer8TextPartE_t>.strings))
        return static_cast<void*>(this);
    return MessagePart::qt_metacast(_clname);
}

int MessageComposer::TextPart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MessagePart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void MessageComposer::TextPart::wordWrappingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MessageComposer::TextPart::warnBadCharsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MessageComposer::TextPart::cleanPlainTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MessageComposer::TextPart::wrappedPlainTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MessageComposer::TextPart::cleanHtmlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MessageComposer::TextPart::embeddedImagesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
