/*
    This file is part of KMail's plugin interface.
    SPDX-FileCopyrightText: 2004 Marc Mutz <mutz@kde.org>
    SPDX-FileCopyrightText: 2004 Ingo Kloecker <kloecker@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "bodypart.h"

MimeTreeParser::Interface::BodyPartMemento::~BodyPartMemento() = default;

MimeTreeParser::Interface::BodyPart::~BodyPart() = default;
