/****************************************************************************
** Meta object code from reading C++ file 'searchlinecommand.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagelist/src/core/widgets/searchlinecommand.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'searchlinecommand.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11MessageList4Core17SearchLineCommandE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageList::Core::SearchLineCommand::qt_create_metaobjectdata<qt_meta_tag_ZN11MessageList4Core17SearchLineCommandE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageList::Core::SearchLineCommand",
        "SearchLineType",
        "Unknown",
        "HasStateOrAttachment",
        "Literal",
        "To",
        "Bcc",
        "Cc",
        "From",
        "Subject",
        "Body",
        "Date",
        "Size",
        "HasAttachment",
        "HasInvitation",
        "IsImportant",
        "IsRead",
        "IsUnRead",
        "IsIgnored",
        "IsHam",
        "IsSpam",
        "IsWatched",
        "IsReplied",
        "IsForwarded",
        "IsEncrypted",
        "IsQueued",
        "IsSent",
        "IsDeleted",
        "IsAction",
        "Category",
        "Larger",
        "Smaller",
        "OlderThan",
        "NewerThan"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SearchLineType'
        QtMocHelpers::EnumData<enum SearchLineType>(1, 1, QMC::EnumFlags{}).add({
            {    2, SearchLineType::Unknown },
            {    3, SearchLineType::HasStateOrAttachment },
            {    4, SearchLineType::Literal },
            {    5, SearchLineType::To },
            {    6, SearchLineType::Bcc },
            {    7, SearchLineType::Cc },
            {    8, SearchLineType::From },
            {    9, SearchLineType::Subject },
            {   10, SearchLineType::Body },
            {   11, SearchLineType::Date },
            {   12, SearchLineType::Size },
            {   13, SearchLineType::HasAttachment },
            {   14, SearchLineType::HasInvitation },
            {   15, SearchLineType::IsImportant },
            {   16, SearchLineType::IsRead },
            {   17, SearchLineType::IsUnRead },
            {   18, SearchLineType::IsIgnored },
            {   19, SearchLineType::IsHam },
            {   20, SearchLineType::IsSpam },
            {   21, SearchLineType::IsWatched },
            {   22, SearchLineType::IsReplied },
            {   23, SearchLineType::IsForwarded },
            {   24, SearchLineType::IsEncrypted },
            {   25, SearchLineType::IsQueued },
            {   26, SearchLineType::IsSent },
            {   27, SearchLineType::IsDeleted },
            {   28, SearchLineType::IsAction },
            {   29, SearchLineType::Category },
            {   30, SearchLineType::Larger },
            {   31, SearchLineType::Smaller },
            {   32, SearchLineType::OlderThan },
            {   33, SearchLineType::NewerThan },
        }),
    };
    return QtMocHelpers::metaObjectData<SearchLineCommand, qt_meta_tag_ZN11MessageList4Core17SearchLineCommandE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageList::Core::SearchLineCommand::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4Core17SearchLineCommandE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4Core17SearchLineCommandE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11MessageList4Core17SearchLineCommandE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
