/*
  SPDX-FileCopyrightText: 2009 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.net
  SPDX-FileCopyrightText: 2009 Andras Mantia <andras@kdab.net>

  SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "objecttreesource.h"

using namespace MimeTreeParser;

Interface::ObjectTreeSource::ObjectTreeSource() = default;

Interface::ObjectTreeSource::~ObjectTreeSource() = default;
