/********************************************************************************
** Form generated from reading UI file 'attachmentpropertiesdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ATTACHMENTPROPERTIESDIALOG_H
#define UI_ATTACHMENTPROPERTIESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_AttachmentPropertiesDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *mimeType;
    QLabel *mimeIcon;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_2;
    QLabel *size;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_3;
    QLineEdit *name;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_4;
    QLineEdit *description;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_5;
    QComboBox *encoding;
    QCheckBox *autoDisplay;
    QCheckBox *encrypt;
    QCheckBox *sign;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *AttachmentPropertiesDialog)
    {
        if (AttachmentPropertiesDialog->objectName().isEmpty())
            AttachmentPropertiesDialog->setObjectName("AttachmentPropertiesDialog");
        AttachmentPropertiesDialog->resize(371, 252);
        verticalLayout = new QVBoxLayout(AttachmentPropertiesDialog);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(AttachmentPropertiesDialog);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(label);


        gridLayout->addLayout(horizontalLayout, 0, 0, 1, 1);

        mimeType = new QComboBox(AttachmentPropertiesDialog);
        mimeType->setObjectName("mimeType");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mimeType->sizePolicy().hasHeightForWidth());
        mimeType->setSizePolicy(sizePolicy1);
        mimeType->setEditable(true);

        gridLayout->addWidget(mimeType, 0, 1, 1, 1);

        mimeIcon = new QLabel(AttachmentPropertiesDialog);
        mimeIcon->setObjectName("mimeIcon");
        mimeIcon->setText(QString::fromUtf8("pretty icon"));

        gridLayout->addWidget(mimeIcon, 0, 2, 2, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_2 = new QLabel(AttachmentPropertiesDialog);
        label_2->setObjectName("label_2");
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(label_2);


        gridLayout->addLayout(horizontalLayout_2, 1, 0, 1, 1);

        size = new QLabel(AttachmentPropertiesDialog);
        size->setObjectName("size");
        size->setText(QString::fromUtf8("size goes here"));

        gridLayout->addWidget(size, 1, 1, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_3 = new QLabel(AttachmentPropertiesDialog);
        label_3->setObjectName("label_3");
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_3->addWidget(label_3);


        gridLayout->addLayout(horizontalLayout_3, 2, 0, 1, 1);

        name = new QLineEdit(AttachmentPropertiesDialog);
        name->setObjectName("name");
        name->setClearButtonEnabled(false);

        gridLayout->addWidget(name, 2, 1, 1, 2);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_4 = new QLabel(AttachmentPropertiesDialog);
        label_4->setObjectName("label_4");
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_4->addWidget(label_4);


        gridLayout->addLayout(horizontalLayout_4, 3, 0, 1, 1);

        description = new QLineEdit(AttachmentPropertiesDialog);
        description->setObjectName("description");
        description->setClearButtonEnabled(true);

        gridLayout->addWidget(description, 3, 1, 1, 2);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_5 = new QLabel(AttachmentPropertiesDialog);
        label_5->setObjectName("label_5");
        sizePolicy.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy);
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_5->addWidget(label_5);


        gridLayout->addLayout(horizontalLayout_5, 4, 0, 1, 1);

        encoding = new QComboBox(AttachmentPropertiesDialog);
        encoding->setObjectName("encoding");

        gridLayout->addWidget(encoding, 4, 1, 1, 2);

        autoDisplay = new QCheckBox(AttachmentPropertiesDialog);
        autoDisplay->setObjectName("autoDisplay");

        gridLayout->addWidget(autoDisplay, 5, 1, 1, 2);

        encrypt = new QCheckBox(AttachmentPropertiesDialog);
        encrypt->setObjectName("encrypt");

        gridLayout->addWidget(encrypt, 6, 1, 1, 2);

        sign = new QCheckBox(AttachmentPropertiesDialog);
        sign->setObjectName("sign");

        gridLayout->addWidget(sign, 7, 1, 1, 2);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 31, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        label->setBuddy(mimeType);
        label_3->setBuddy(name);
        label_4->setBuddy(description);
        label_5->setBuddy(encoding);
#endif // QT_CONFIG(shortcut)

        retranslateUi(AttachmentPropertiesDialog);

        QMetaObject::connectSlotsByName(AttachmentPropertiesDialog);
    } // setupUi

    void retranslateUi(QWidget *AttachmentPropertiesDialog)
    {
        label->setText(tr2i18n("Type:", "MIME type of this attachment"));
        label_2->setText(tr2i18n("Size:", "size of the attachment"));
        label_3->setText(tr2i18n("Name:", "file name of the attachment"));
        label_4->setText(tr2i18n("Description:", "description of the attachment"));
        label_5->setText(tr2i18n("Encoding:", "encoding of the attachment"));
        autoDisplay->setText(tr2i18n("Suggest automatic display", nullptr));
        encrypt->setText(tr2i18n("Encrypt this attachment", nullptr));
        sign->setText(tr2i18n("Sign this attachment", nullptr));
        (void)AttachmentPropertiesDialog;
    } // retranslateUi

};

namespace Ui {
    class AttachmentPropertiesDialog: public Ui_AttachmentPropertiesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // ATTACHMENTPROPERTIESDIALOG_H

