// This file is generated by kconfig_compiler_kf6 from defaultgrantleeheaderstyleplugin.kcfg.
// All changes you do to this file will be lost.

#include "globalsettings_base.h"

#include <qglobal.h>
#include <QFile>

class DefaultGrantleeHeaderStylePluginSettingsHelper
{
  public:
    DefaultGrantleeHeaderStylePluginSettingsHelper() noexcept : q(nullptr) {}
    ~DefaultGrantleeHeaderStylePluginSettingsHelper() noexcept { delete q; q = nullptr; }
    DefaultGrantleeHeaderStylePluginSettingsHelper(const DefaultGrantleeHeaderStylePluginSettingsHelper&) = delete;
    DefaultGrantleeHeaderStylePluginSettingsHelper& operator=(const DefaultGrantleeHeaderStylePluginSettingsHelper&) = delete;
    DefaultGrantleeHeaderStylePluginSettings *q;
};
Q_GLOBAL_STATIC(DefaultGrantleeHeaderStylePluginSettingsHelper, s_globalDefaultGrantleeHeaderStylePluginSettings)
DefaultGrantleeHeaderStylePluginSettings *DefaultGrantleeHeaderStylePluginSettings::self()
{
  if (!s_globalDefaultGrantleeHeaderStylePluginSettings()->q) {
    new DefaultGrantleeHeaderStylePluginSettings;
    s_globalDefaultGrantleeHeaderStylePluginSettings()->q->read();
  }

  return s_globalDefaultGrantleeHeaderStylePluginSettings()->q;
}

DefaultGrantleeHeaderStylePluginSettings::DefaultGrantleeHeaderStylePluginSettings( )
  : KConfigSkeleton()
{
  Q_ASSERT(!s_globalDefaultGrantleeHeaderStylePluginSettings()->q);
  s_globalDefaultGrantleeHeaderStylePluginSettings()->q = this;
  setCurrentGroup( QStringLiteral( "GrantleeDefaultTheme" ) );

  mThemeNameItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "ThemeName" ), mThemeName, QStringLiteral( "5.2" ) );
  addItem( mThemeNameItem, QStringLiteral( "ThemeName" ) );
}

DefaultGrantleeHeaderStylePluginSettings::~DefaultGrantleeHeaderStylePluginSettings()
{
  if (s_globalDefaultGrantleeHeaderStylePluginSettings.exists() && !s_globalDefaultGrantleeHeaderStylePluginSettings.isDestroyed()) {
    s_globalDefaultGrantleeHeaderStylePluginSettings()->q = nullptr;
  }
}

