/****************************************************************************
** Meta object code from reading C++ file 'pane.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagelist/src/pane.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pane.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11MessageList4PaneE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageList::Pane::qt_create_metaobjectdata<qt_meta_tag_ZN11MessageList4PaneE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageList::Pane",
        "messageSelected",
        "",
        "Akonadi::Item",
        "item",
        "messageActivated",
        "selectionChanged",
        "messageExpanded",
        "messageCollapsed",
        "messageStatusChangeRequest",
        "Akonadi::MessageStatus",
        "set",
        "clear",
        "statusMessage",
        "message",
        "currentTabChanged",
        "forceLostFocus",
        "selectAll",
        "sortOrderMenuAboutToShow",
        "aggregationMenuAboutToShow",
        "themeMenuAboutToShow"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'messageSelected'
        QtMocHelpers::SignalData<void(const Akonadi::Item &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'messageActivated'
        QtMocHelpers::SignalData<void(const Akonadi::Item &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'selectionChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'messageExpanded'
        QtMocHelpers::SignalData<void(const Akonadi::Item &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'messageCollapsed'
        QtMocHelpers::SignalData<void(const Akonadi::Item &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'messageStatusChangeRequest'
        QtMocHelpers::SignalData<void(const Akonadi::Item &, const Akonadi::MessageStatus &, const Akonadi::MessageStatus &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 10, 11 }, { 0x80000000 | 10, 12 },
        }}),
        // Signal 'statusMessage'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'currentTabChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'forceLostFocus'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sortOrderMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'aggregationMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'themeMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Pane, qt_meta_tag_ZN11MessageList4PaneE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageList::Pane::staticMetaObject = { {
    QMetaObject::SuperData::link<QTabWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4PaneE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4PaneE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11MessageList4PaneE_t>.metaTypes,
    nullptr
} };

void MessageList::Pane::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Pane *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->messageSelected((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1]))); break;
        case 1: _t->messageActivated((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1]))); break;
        case 2: _t->selectionChanged(); break;
        case 3: _t->messageExpanded((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1]))); break;
        case 4: _t->messageCollapsed((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1]))); break;
        case 5: _t->messageStatusChangeRequest((*reinterpret_cast<std::add_pointer_t<Akonadi::Item>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Akonadi::MessageStatus>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Akonadi::MessageStatus>>(_a[3]))); break;
        case 6: _t->statusMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->currentTabChanged(); break;
        case 8: _t->forceLostFocus(); break;
        case 9: _t->selectAll(); break;
        case 10: _t->sortOrderMenuAboutToShow(); break;
        case 11: _t->aggregationMenuAboutToShow(); break;
        case 12: _t->themeMenuAboutToShow(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Pane::*)(const Akonadi::Item & )>(_a, &Pane::messageSelected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Pane::*)(const Akonadi::Item & )>(_a, &Pane::messageActivated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Pane::*)()>(_a, &Pane::selectionChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Pane::*)(const Akonadi::Item & )>(_a, &Pane::messageExpanded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Pane::*)(const Akonadi::Item & )>(_a, &Pane::messageCollapsed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Pane::*)(const Akonadi::Item & , const Akonadi::MessageStatus & , const Akonadi::MessageStatus & )>(_a, &Pane::messageStatusChangeRequest, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Pane::*)(const QString & )>(_a, &Pane::statusMessage, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Pane::*)()>(_a, &Pane::currentTabChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Pane::*)()>(_a, &Pane::forceLostFocus, 8))
            return;
    }
}

const QMetaObject *MessageList::Pane::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageList::Pane::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4PaneE_t>.strings))
        return static_cast<void*>(this);
    return QTabWidget::qt_metacast(_clname);
}

int MessageList::Pane::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTabWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void MessageList::Pane::messageSelected(const Akonadi::Item & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MessageList::Pane::messageActivated(const Akonadi::Item & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MessageList::Pane::selectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MessageList::Pane::messageExpanded(const Akonadi::Item & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MessageList::Pane::messageCollapsed(const Akonadi::Item & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MessageList::Pane::messageStatusChangeRequest(const Akonadi::Item & _t1, const Akonadi::MessageStatus & _t2, const Akonadi::MessageStatus & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void MessageList::Pane::statusMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void MessageList::Pane::currentTabChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void MessageList::Pane::forceLostFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
