/****************************************************************************
** Meta object code from reading C++ file 'recipientspicker.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagecomposer/src/recipient/recipientspicker.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'recipientspicker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15MessageComposer16RecipientsPickerE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::RecipientsPicker::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer16RecipientsPickerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::RecipientsPicker",
        "pickedRecipient",
        "",
        "Recipient",
        "bool&",
        "slotToClicked",
        "slotCcClicked",
        "slotBccClicked",
        "slotReplyToClicked",
        "slotPicked",
        "slotSearchLDAP",
        "ldapSearchResult",
        "slotSelectionChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pickedRecipient'
        QtMocHelpers::SignalData<void(const Recipient &, bool &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 },
        }}),
        // Slot 'slotToClicked'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCcClicked'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotBccClicked'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotReplyToClicked'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotPicked'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSearchLDAP'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'ldapSearchResult'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RecipientsPicker, qt_meta_tag_ZN15MessageComposer16RecipientsPickerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::RecipientsPicker::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer16RecipientsPickerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer16RecipientsPickerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer16RecipientsPickerE_t>.metaTypes,
    nullptr
} };

void MessageComposer::RecipientsPicker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RecipientsPicker *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pickedRecipient((*reinterpret_cast<std::add_pointer_t<Recipient>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[2]))); break;
        case 1: _t->slotToClicked(); break;
        case 2: _t->slotCcClicked(); break;
        case 3: _t->slotBccClicked(); break;
        case 4: _t->slotReplyToClicked(); break;
        case 5: _t->slotPicked(); break;
        case 6: _t->slotSearchLDAP(); break;
        case 7: _t->ldapSearchResult(); break;
        case 8: _t->slotSelectionChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RecipientsPicker::*)(const Recipient & , bool & )>(_a, &RecipientsPicker::pickedRecipient, 0))
            return;
    }
}

const QMetaObject *MessageComposer::RecipientsPicker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::RecipientsPicker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer16RecipientsPickerE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int MessageComposer::RecipientsPicker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void MessageComposer::RecipientsPicker::pickedRecipient(const Recipient & _t1, bool & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
