/*
   SPDX-FileCopyrightText: 2016-2026 Laurent Montel <montel@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/
#pragma once

#include <QString>

namespace WebEngineViewer
{
namespace WebEngineAccessKeyUtils
{
[[nodiscard]] QString accessKeyScript();
}
}
