/****************************************************************************
** Meta object code from reading C++ file 'sortorder.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagelist/src/core/sortorder.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sortorder.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11MessageList4Core9SortOrderE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageList::Core::SortOrder::qt_create_metaobjectdata<qt_meta_tag_ZN11MessageList4Core9SortOrderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageList::Core::SortOrder",
        "GroupSorting",
        "NoGroupSorting",
        "SortGroupsByDateTime",
        "SortGroupsByDateTimeOfMostRecent",
        "SortGroupsBySenderOrReceiver",
        "SortGroupsBySender",
        "SortGroupsByReceiver",
        "SortDirection",
        "Ascending",
        "Descending",
        "MessageSorting",
        "NoMessageSorting",
        "SortMessagesByDateTime",
        "SortMessagesByDateTimeOfMostRecent",
        "SortMessagesBySenderOrReceiver",
        "SortMessagesBySender",
        "SortMessagesByReceiver",
        "SortMessagesBySubject",
        "SortMessagesBySize",
        "SortMessagesByActionItemStatus",
        "SortMessagesByUnreadStatus",
        "SortMessagesByImportantStatus",
        "SortMessagesByAttachmentStatus"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'GroupSorting'
        QtMocHelpers::EnumData<enum GroupSorting>(1, 1, QMC::EnumFlags{}).add({
            {    2, GroupSorting::NoGroupSorting },
            {    3, GroupSorting::SortGroupsByDateTime },
            {    4, GroupSorting::SortGroupsByDateTimeOfMostRecent },
            {    5, GroupSorting::SortGroupsBySenderOrReceiver },
            {    6, GroupSorting::SortGroupsBySender },
            {    7, GroupSorting::SortGroupsByReceiver },
        }),
        // enum 'SortDirection'
        QtMocHelpers::EnumData<enum SortDirection>(8, 8, QMC::EnumFlags{}).add({
            {    9, SortDirection::Ascending },
            {   10, SortDirection::Descending },
        }),
        // enum 'MessageSorting'
        QtMocHelpers::EnumData<enum MessageSorting>(11, 11, QMC::EnumFlags{}).add({
            {   12, MessageSorting::NoMessageSorting },
            {   13, MessageSorting::SortMessagesByDateTime },
            {   14, MessageSorting::SortMessagesByDateTimeOfMostRecent },
            {   15, MessageSorting::SortMessagesBySenderOrReceiver },
            {   16, MessageSorting::SortMessagesBySender },
            {   17, MessageSorting::SortMessagesByReceiver },
            {   18, MessageSorting::SortMessagesBySubject },
            {   19, MessageSorting::SortMessagesBySize },
            {   20, MessageSorting::SortMessagesByActionItemStatus },
            {   21, MessageSorting::SortMessagesByUnreadStatus },
            {   22, MessageSorting::SortMessagesByImportantStatus },
            {   23, MessageSorting::SortMessagesByAttachmentStatus },
        }),
    };
    return QtMocHelpers::metaObjectData<SortOrder, qt_meta_tag_ZN11MessageList4Core9SortOrderE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageList::Core::SortOrder::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4Core9SortOrderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4Core9SortOrderE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11MessageList4Core9SortOrderE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
