/****************************************************************************
** Meta object code from reading C++ file 'itippart.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagecomposer/src/part/itippart.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'itippart.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15MessageComposer8ItipPartE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::ItipPart::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer8ItipPartE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::ItipPart",
        "methodChanged",
        "",
        "invitationChanged",
        "invitationBodyChanged",
        "outlookConformInvitationChanged",
        "invitation",
        "invitationBody",
        "outlookConformInvitation",
        "method"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'methodChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'invitationChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'invitationBodyChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'outlookConformInvitationChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'invitation'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'invitationBody'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'outlookConformInvitation'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'method'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ItipPart, qt_meta_tag_ZN15MessageComposer8ItipPartE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::ItipPart::staticMetaObject = { {
    QMetaObject::SuperData::link<MessagePart::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer8ItipPartE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer8ItipPartE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer8ItipPartE_t>.metaTypes,
    nullptr
} };

void MessageComposer::ItipPart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ItipPart *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->methodChanged(); break;
        case 1: _t->invitationChanged(); break;
        case 2: _t->invitationBodyChanged(); break;
        case 3: _t->outlookConformInvitationChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ItipPart::*)()>(_a, &ItipPart::methodChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ItipPart::*)()>(_a, &ItipPart::invitationChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ItipPart::*)()>(_a, &ItipPart::invitationBodyChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ItipPart::*)()>(_a, &ItipPart::outlookConformInvitationChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->invitation(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->invitationBody(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->outlookConformInvitation(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->method(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setInvitation(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setInvitationBody(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setOutlookConformInvitation(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setMethod(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *MessageComposer::ItipPart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::ItipPart::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer8ItipPartE_t>.strings))
        return static_cast<void*>(this);
    return MessagePart::qt_metacast(_clname);
}

int MessageComposer::ItipPart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MessagePart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void MessageComposer::ItipPart::methodChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MessageComposer::ItipPart::invitationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MessageComposer::ItipPart::invitationBodyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MessageComposer::ItipPart::outlookConformInvitationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
