/****************************************************************************
** Meta object code from reading C++ file 'attachmentmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagecomposer/src/attachment/attachmentmodel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'attachmentmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15MessageComposer15AttachmentModelE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposer::AttachmentModel::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageComposer15AttachmentModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposer::AttachmentModel",
        "encryptEnabled",
        "",
        "enabled",
        "signEnabled",
        "autoDisplayEnabled",
        "attachUrlsRequested",
        "QList<QUrl>",
        "urls",
        "attachItemsRequester",
        "Akonadi::Item::List",
        "attachmentRemoved",
        "MessageCore::AttachmentPart::Ptr",
        "part",
        "attachmentCompressRequested",
        "compress"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'encryptEnabled'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'signEnabled'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'autoDisplayEnabled'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'attachUrlsRequested'
        QtMocHelpers::SignalData<void(const QList<QUrl> &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'attachItemsRequester'
        QtMocHelpers::SignalData<void(const Akonadi::Item::List &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Signal 'attachmentRemoved'
        QtMocHelpers::SignalData<void(MessageCore::AttachmentPart::Ptr)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'attachmentCompressRequested'
        QtMocHelpers::SignalData<void(MessageCore::AttachmentPart::Ptr, bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { QMetaType::Bool, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AttachmentModel, qt_meta_tag_ZN15MessageComposer15AttachmentModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposer::AttachmentModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer15AttachmentModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer15AttachmentModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageComposer15AttachmentModelE_t>.metaTypes,
    nullptr
} };

void MessageComposer::AttachmentModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AttachmentModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->encryptEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->signEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->autoDisplayEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->attachUrlsRequested((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 4: _t->attachItemsRequester((*reinterpret_cast<std::add_pointer_t<Akonadi::Item::List>>(_a[1]))); break;
        case 5: _t->attachmentRemoved((*reinterpret_cast<std::add_pointer_t<MessageCore::AttachmentPart::Ptr>>(_a[1]))); break;
        case 6: _t->attachmentCompressRequested((*reinterpret_cast<std::add_pointer_t<MessageCore::AttachmentPart::Ptr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akonadi::Item::List >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageCore::AttachmentPart::Ptr >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageCore::AttachmentPart::Ptr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AttachmentModel::*)(bool )>(_a, &AttachmentModel::encryptEnabled, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AttachmentModel::*)(bool )>(_a, &AttachmentModel::signEnabled, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AttachmentModel::*)(bool )>(_a, &AttachmentModel::autoDisplayEnabled, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AttachmentModel::*)(const QList<QUrl> & )>(_a, &AttachmentModel::attachUrlsRequested, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AttachmentModel::*)(const Akonadi::Item::List & )>(_a, &AttachmentModel::attachItemsRequester, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AttachmentModel::*)(MessageCore::AttachmentPart::Ptr )>(_a, &AttachmentModel::attachmentRemoved, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AttachmentModel::*)(MessageCore::AttachmentPart::Ptr , bool )>(_a, &AttachmentModel::attachmentCompressRequested, 6))
            return;
    }
}

const QMetaObject *MessageComposer::AttachmentModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposer::AttachmentModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageComposer15AttachmentModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int MessageComposer::AttachmentModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void MessageComposer::AttachmentModel::encryptEnabled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MessageComposer::AttachmentModel::signEnabled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MessageComposer::AttachmentModel::autoDisplayEnabled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MessageComposer::AttachmentModel::attachUrlsRequested(const QList<QUrl> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MessageComposer::AttachmentModel::attachItemsRequester(const Akonadi::Item::List & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MessageComposer::AttachmentModel::attachmentRemoved(MessageCore::AttachmentPart::Ptr _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void MessageComposer::AttachmentModel::attachmentCompressRequested(MessageCore::AttachmentPart::Ptr _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}
QT_WARNING_POP
