/********************************************************************************
** Form generated from reading UI file 'invitationsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INVITATIONSETTINGS_H
#define UI_INVITATIONSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_InvitationSettings
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QCheckBox *mLegacyMangleFromTo;
    QCheckBox *mLegacyBodyInvites;
    QCheckBox *mExchangeCompatibleInvitations;
    QCheckBox *mAutomaticSending;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout;
    QCheckBox *mDeleteInvitations;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *InvitationSettings)
    {
        if (InvitationSettings->objectName().isEmpty())
            InvitationSettings->setObjectName("InvitationSettings");
        InvitationSettings->resize(572, 297);
        verticalLayout_2 = new QVBoxLayout(InvitationSettings);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox_2 = new QGroupBox(InvitationSettings);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setObjectName("verticalLayout");
        mLegacyMangleFromTo = new QCheckBox(groupBox_2);
        mLegacyMangleFromTo->setObjectName("mLegacyMangleFromTo");

        verticalLayout->addWidget(mLegacyMangleFromTo);

        mLegacyBodyInvites = new QCheckBox(groupBox_2);
        mLegacyBodyInvites->setObjectName("mLegacyBodyInvites");

        verticalLayout->addWidget(mLegacyBodyInvites);

        mExchangeCompatibleInvitations = new QCheckBox(groupBox_2);
        mExchangeCompatibleInvitations->setObjectName("mExchangeCompatibleInvitations");

        verticalLayout->addWidget(mExchangeCompatibleInvitations);

        mAutomaticSending = new QCheckBox(groupBox_2);
        mAutomaticSending->setObjectName("mAutomaticSending");

        verticalLayout->addWidget(mAutomaticSending);


        verticalLayout_2->addWidget(groupBox_2);

        groupBox_3 = new QGroupBox(InvitationSettings);
        groupBox_3->setObjectName("groupBox_3");
        gridLayout = new QGridLayout(groupBox_3);
        gridLayout->setObjectName("gridLayout");
        mDeleteInvitations = new QCheckBox(groupBox_3);
        mDeleteInvitations->setObjectName("mDeleteInvitations");

        gridLayout->addWidget(mDeleteInvitations, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 1, 0, 1, 1);


        verticalLayout_2->addWidget(groupBox_3);


        retranslateUi(InvitationSettings);

        QMetaObject::connectSlotsByName(InvitationSettings);
    } // setupUi

    void retranslateUi(QWidget *InvitationSettings)
    {
        groupBox_2->setTitle(tr2i18n("Groupware Compatibility && Legacy Options", nullptr));
#if QT_CONFIG(tooltip)
        mLegacyMangleFromTo->setToolTip(tr2i18n("Turn this option on in order to make Outlook(tm) understand your answers to invitation replies", nullptr));
#endif // QT_CONFIG(tooltip)
        mLegacyMangleFromTo->setText(tr2i18n("&Mangle From:/To: headers in replies to invitations", nullptr));
#if QT_CONFIG(tooltip)
        mLegacyBodyInvites->setToolTip(tr2i18n("Turn this option on in order to make Outlook(tm) understand your answers to invitations", nullptr));
#endif // QT_CONFIG(tooltip)
        mLegacyBodyInvites->setText(tr2i18n("Send &invitations in the mail body", nullptr));
#if QT_CONFIG(tooltip)
        mExchangeCompatibleInvitations->setToolTip(tr2i18n("Microsoft Outlook, when used in combination with a Microsoft Exchange server,\n"
"has a problem understanding standards-compliant groupware email.\n"
"Turn this option on to send groupware invitations in a way that Microsoft Exchange understands.", nullptr));
#endif // QT_CONFIG(tooltip)
        mExchangeCompatibleInvitations->setText(tr2i18n("Exchange-compatible invitation &naming", nullptr));
#if QT_CONFIG(tooltip)
        mAutomaticSending->setToolTip(tr2i18n("When this is on, the user will not see the mail composer window. Invitation mails are sent automatically.", nullptr));
#endif // QT_CONFIG(tooltip)
        mAutomaticSending->setText(tr2i18n("&Automatic invitation sending", nullptr));
        groupBox_3->setTitle(tr2i18n("Options", nullptr));
        mDeleteInvitations->setText(tr2i18n("Delete invitation emails after the reply to them has been sent", nullptr));
        (void)InvitationSettings;
    } // retranslateUi

};

namespace Ui {
    class InvitationSettings: public Ui_InvitationSettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // INVITATIONSETTINGS_H

