/********************************************************************************
** Form generated from reading UI file 'imagescalingwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMAGESCALINGWIDGET_H
#define UI_IMAGESCALINGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "imagescaling/imagescalingselectformat.h"

QT_BEGIN_NAMESPACE

class Ui_ImageScalingWidget
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *enabledAutoResize;
    QTabWidget *tabWidget;
    QWidget *tab;
    QGridLayout *gridLayout_4;
    QCheckBox *KeepImageRatio;
    QCheckBox *AskBeforeResizing;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *WriteToImageFormat;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *ReduceImageToMaximum;
    QFormLayout *formLayout;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_2;
    QLabel *label_3;
    QGridLayout *gridLayout_2;
    QComboBox *CBMaximumWidth;
    QComboBox *CBMaximumHeight;
    QSpinBox *customMaximumWidth;
    QSpinBox *customMaximumHeight;
    QCheckBox *EnlargeImageToMinimum;
    QFormLayout *formLayout_2;
    QVBoxLayout *verticalLayout_5;
    QLabel *label_4;
    QLabel *label_5;
    QGridLayout *gridLayout_3;
    QComboBox *CBMinimumWidth;
    QComboBox *CBMinimumHeight;
    QSpinBox *customMinimumWidth;
    QSpinBox *customMinimumHeight;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *skipImageSizeLower;
    QSpinBox *imageSize;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *filterSourceGroupBox;
    QVBoxLayout *verticalLayout_6;
    QRadioButton *notFilterFilename;
    QRadioButton *includeFilesWithPattern;
    QRadioButton *excludeFilesWithPattern;
    QHBoxLayout *horizontalLayout_3;
    QLabel *patternLabel;
    QLineEdit *pattern;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_9;
    QCheckBox *resizeImageWithFormats;
    MessageComposer::ImageScalingSelectFormat *resizeImageWithFormatsType;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QSpacerItem *verticalSpacer_3;
    QLineEdit *renameResizedImagePattern;
    QCheckBox *renameResizedImage;
    QLabel *help;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout_8;
    QRadioButton *doNotFilterRecipients;
    QRadioButton *resizeEachEmails;
    QRadioButton *resizeOneEmails;
    QLineEdit *resizeEmailsPattern;
    QRadioButton *doNotResizeEachEmails;
    QRadioButton *doNotResizeOneEmails;
    QLineEdit *doNotResizePattern;
    QSpacerItem *verticalSpacer_4;

    void setupUi(QWidget *ImageScalingWidget)
    {
        if (ImageScalingWidget->objectName().isEmpty())
            ImageScalingWidget->setObjectName("ImageScalingWidget");
        ImageScalingWidget->resize(614, 453);
        verticalLayout = new QVBoxLayout(ImageScalingWidget);
        verticalLayout->setObjectName("verticalLayout");
        enabledAutoResize = new QCheckBox(ImageScalingWidget);
        enabledAutoResize->setObjectName("enabledAutoResize");

        verticalLayout->addWidget(enabledAutoResize);

        tabWidget = new QTabWidget(ImageScalingWidget);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        gridLayout_4 = new QGridLayout(tab);
        gridLayout_4->setObjectName("gridLayout_4");
        KeepImageRatio = new QCheckBox(tab);
        KeepImageRatio->setObjectName("KeepImageRatio");

        gridLayout_4->addWidget(KeepImageRatio, 0, 0, 1, 1);

        AskBeforeResizing = new QCheckBox(tab);
        AskBeforeResizing->setObjectName("AskBeforeResizing");

        gridLayout_4->addWidget(AskBeforeResizing, 1, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(tab);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        WriteToImageFormat = new QComboBox(tab);
        WriteToImageFormat->setObjectName("WriteToImageFormat");

        horizontalLayout->addWidget(WriteToImageFormat);


        gridLayout_4->addLayout(horizontalLayout, 2, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(350, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer, 2, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_4->addItem(verticalSpacer, 3, 1, 1, 1);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_2 = new QVBoxLayout(tab_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        ReduceImageToMaximum = new QCheckBox(tab_2);
        ReduceImageToMaximum->setObjectName("ReduceImageToMaximum");

        verticalLayout_2->addWidget(ReduceImageToMaximum);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        label_2 = new QLabel(tab_2);
        label_2->setObjectName("label_2");

        verticalLayout_4->addWidget(label_2);

        label_3 = new QLabel(tab_2);
        label_3->setObjectName("label_3");

        verticalLayout_4->addWidget(label_3);


        formLayout->setLayout(0, QFormLayout::ItemRole::LabelRole, verticalLayout_4);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        CBMaximumWidth = new QComboBox(tab_2);
        CBMaximumWidth->setObjectName("CBMaximumWidth");

        gridLayout_2->addWidget(CBMaximumWidth, 0, 0, 1, 1);

        CBMaximumHeight = new QComboBox(tab_2);
        CBMaximumHeight->setObjectName("CBMaximumHeight");

        gridLayout_2->addWidget(CBMaximumHeight, 2, 0, 1, 1);

        customMaximumWidth = new QSpinBox(tab_2);
        customMaximumWidth->setObjectName("customMaximumWidth");
        customMaximumWidth->setMinimum(1);
        customMaximumWidth->setMaximum(999999);

        gridLayout_2->addWidget(customMaximumWidth, 0, 1, 1, 1);

        customMaximumHeight = new QSpinBox(tab_2);
        customMaximumHeight->setObjectName("customMaximumHeight");
        customMaximumHeight->setMinimum(1);
        customMaximumHeight->setMaximum(999999);

        gridLayout_2->addWidget(customMaximumHeight, 2, 1, 1, 1);


        formLayout->setLayout(0, QFormLayout::ItemRole::FieldRole, gridLayout_2);


        verticalLayout_2->addLayout(formLayout);

        EnlargeImageToMinimum = new QCheckBox(tab_2);
        EnlargeImageToMinimum->setObjectName("EnlargeImageToMinimum");

        verticalLayout_2->addWidget(EnlargeImageToMinimum);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        label_4 = new QLabel(tab_2);
        label_4->setObjectName("label_4");

        verticalLayout_5->addWidget(label_4);

        label_5 = new QLabel(tab_2);
        label_5->setObjectName("label_5");

        verticalLayout_5->addWidget(label_5);


        formLayout_2->setLayout(0, QFormLayout::ItemRole::LabelRole, verticalLayout_5);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        CBMinimumWidth = new QComboBox(tab_2);
        CBMinimumWidth->setObjectName("CBMinimumWidth");

        gridLayout_3->addWidget(CBMinimumWidth, 0, 0, 1, 1);

        CBMinimumHeight = new QComboBox(tab_2);
        CBMinimumHeight->setObjectName("CBMinimumHeight");

        gridLayout_3->addWidget(CBMinimumHeight, 2, 0, 1, 1);

        customMinimumWidth = new QSpinBox(tab_2);
        customMinimumWidth->setObjectName("customMinimumWidth");
        customMinimumWidth->setMinimum(1);
        customMinimumWidth->setMaximum(999999);

        gridLayout_3->addWidget(customMinimumWidth, 0, 1, 1, 1);

        customMinimumHeight = new QSpinBox(tab_2);
        customMinimumHeight->setObjectName("customMinimumHeight");
        customMinimumHeight->setMinimum(1);
        customMinimumHeight->setMaximum(999999);

        gridLayout_3->addWidget(customMinimumHeight, 2, 1, 1, 1);


        formLayout_2->setLayout(0, QFormLayout::ItemRole::FieldRole, gridLayout_3);


        verticalLayout_2->addLayout(formLayout_2);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        skipImageSizeLower = new QCheckBox(tab_2);
        skipImageSizeLower->setObjectName("skipImageSizeLower");

        horizontalLayout_2->addWidget(skipImageSizeLower);

        imageSize = new QSpinBox(tab_2);
        imageSize->setObjectName("imageSize");
        imageSize->setMaximum(99999);

        horizontalLayout_2->addWidget(imageSize);


        verticalLayout_2->addLayout(horizontalLayout_2);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_3 = new QVBoxLayout(tab_3);
        verticalLayout_3->setObjectName("verticalLayout_3");
        filterSourceGroupBox = new QGroupBox(tab_3);
        filterSourceGroupBox->setObjectName("filterSourceGroupBox");
        verticalLayout_6 = new QVBoxLayout(filterSourceGroupBox);
        verticalLayout_6->setObjectName("verticalLayout_6");
        notFilterFilename = new QRadioButton(filterSourceGroupBox);
        notFilterFilename->setObjectName("notFilterFilename");
        notFilterFilename->setChecked(true);

        verticalLayout_6->addWidget(notFilterFilename);

        includeFilesWithPattern = new QRadioButton(filterSourceGroupBox);
        includeFilesWithPattern->setObjectName("includeFilesWithPattern");

        verticalLayout_6->addWidget(includeFilesWithPattern);

        excludeFilesWithPattern = new QRadioButton(filterSourceGroupBox);
        excludeFilesWithPattern->setObjectName("excludeFilesWithPattern");

        verticalLayout_6->addWidget(excludeFilesWithPattern);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        patternLabel = new QLabel(filterSourceGroupBox);
        patternLabel->setObjectName("patternLabel");

        horizontalLayout_3->addWidget(patternLabel);

        pattern = new QLineEdit(filterSourceGroupBox);
        pattern->setObjectName("pattern");
        pattern->setProperty("showClearButton", QVariant(true));

        horizontalLayout_3->addWidget(pattern);


        verticalLayout_6->addLayout(horizontalLayout_3);


        verticalLayout_3->addWidget(filterSourceGroupBox);

        groupBox_2 = new QGroupBox(tab_3);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_9 = new QVBoxLayout(groupBox_2);
        verticalLayout_9->setObjectName("verticalLayout_9");
        resizeImageWithFormats = new QCheckBox(groupBox_2);
        resizeImageWithFormats->setObjectName("resizeImageWithFormats");

        verticalLayout_9->addWidget(resizeImageWithFormats);

        resizeImageWithFormatsType = new MessageComposer::ImageScalingSelectFormat(groupBox_2);
        resizeImageWithFormatsType->setObjectName("resizeImageWithFormatsType");

        verticalLayout_9->addWidget(resizeImageWithFormatsType);


        verticalLayout_3->addWidget(groupBox_2);

        groupBox = new QGroupBox(tab_3);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_3, 3, 0, 1, 1);

        renameResizedImagePattern = new QLineEdit(groupBox);
        renameResizedImagePattern->setObjectName("renameResizedImagePattern");
        renameResizedImagePattern->setProperty("showClearButton", QVariant(true));

        gridLayout->addWidget(renameResizedImagePattern, 1, 0, 1, 1);

        renameResizedImage = new QCheckBox(groupBox);
        renameResizedImage->setObjectName("renameResizedImage");

        gridLayout->addWidget(renameResizedImage, 0, 0, 1, 1);

        help = new QLabel(groupBox);
        help->setObjectName("help");

        gridLayout->addWidget(help, 1, 1, 1, 1);


        verticalLayout_3->addWidget(groupBox);

        tabWidget->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        verticalLayout_8 = new QVBoxLayout(tab_4);
        verticalLayout_8->setObjectName("verticalLayout_8");
        doNotFilterRecipients = new QRadioButton(tab_4);
        doNotFilterRecipients->setObjectName("doNotFilterRecipients");
        doNotFilterRecipients->setChecked(true);

        verticalLayout_8->addWidget(doNotFilterRecipients);

        resizeEachEmails = new QRadioButton(tab_4);
        resizeEachEmails->setObjectName("resizeEachEmails");

        verticalLayout_8->addWidget(resizeEachEmails);

        resizeOneEmails = new QRadioButton(tab_4);
        resizeOneEmails->setObjectName("resizeOneEmails");

        verticalLayout_8->addWidget(resizeOneEmails);

        resizeEmailsPattern = new QLineEdit(tab_4);
        resizeEmailsPattern->setObjectName("resizeEmailsPattern");
        resizeEmailsPattern->setProperty("showClearButton", QVariant(true));

        verticalLayout_8->addWidget(resizeEmailsPattern);

        doNotResizeEachEmails = new QRadioButton(tab_4);
        doNotResizeEachEmails->setObjectName("doNotResizeEachEmails");

        verticalLayout_8->addWidget(doNotResizeEachEmails);

        doNotResizeOneEmails = new QRadioButton(tab_4);
        doNotResizeOneEmails->setObjectName("doNotResizeOneEmails");

        verticalLayout_8->addWidget(doNotResizeOneEmails);

        doNotResizePattern = new QLineEdit(tab_4);
        doNotResizePattern->setObjectName("doNotResizePattern");
        doNotResizePattern->setProperty("showClearButton", QVariant(true));

        verticalLayout_8->addWidget(doNotResizePattern);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_8->addItem(verticalSpacer_4);

        tabWidget->addTab(tab_4, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(ImageScalingWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ImageScalingWidget);
    } // setupUi

    void retranslateUi(QWidget *ImageScalingWidget)
    {
        enabledAutoResize->setText(tr2i18n("Automatically resize images", nullptr));
        KeepImageRatio->setText(tr2i18n("Keep ratio", nullptr));
        AskBeforeResizing->setText(tr2i18n("Ask before resizing", nullptr));
        label->setText(tr2i18n("Fallback to format:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), tr2i18n("General", nullptr));
        ReduceImageToMaximum->setText(tr2i18n("Reduce image to maximum dimension:", nullptr));
        label_2->setText(tr2i18n("Maximum width:", nullptr));
        label_3->setText(tr2i18n("Maximum height:", nullptr));
        customMaximumWidth->setSuffix(tr2i18n(" px", nullptr));
        customMaximumHeight->setSuffix(tr2i18n(" px", nullptr));
        EnlargeImageToMinimum->setText(tr2i18n("Enlarge image to minimum dimension", nullptr));
        label_4->setText(tr2i18n("Minimum width:", nullptr));
        label_5->setText(tr2i18n("Minimum height:", nullptr));
        customMinimumWidth->setSuffix(tr2i18n(" px", nullptr));
        customMinimumHeight->setSuffix(tr2i18n(" px", nullptr));
        skipImageSizeLower->setText(tr2i18n("Skip image whose size of the source file is lower than:", nullptr));
        imageSize->setSuffix(tr2i18n(" KiB", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), tr2i18n("Image", nullptr));
        filterSourceGroupBox->setTitle(tr2i18n("Source filter", nullptr));
        notFilterFilename->setText(tr2i18n("Do not filter (All files are included)", nullptr));
        includeFilesWithPattern->setText(tr2i18n("Include all files whose names contains one of the patterns", nullptr));
        excludeFilesWithPattern->setText(tr2i18n("Exclude all files whose names contains one of the patterns", nullptr));
        patternLabel->setText(tr2i18n("Pattern:", nullptr));
#if QT_CONFIG(whatsthis)
        pattern->setWhatsThis(tr2i18n("This line allows to define several pattern. Separate them with ';'.", nullptr));
#endif // QT_CONFIG(whatsthis)
        pattern->setPlaceholderText(tr2i18n("Specify patterns separate with ';'", nullptr));
        groupBox_2->setTitle(tr2i18n("Image Format", nullptr));
        resizeImageWithFormats->setText(tr2i18n("Resize all images with one of these formats", nullptr));
        groupBox->setTitle(tr2i18n("Resize filename", nullptr));
        renameResizedImage->setText(tr2i18n("Rename resized images with following pattern:", nullptr));
        help->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(tab_3), tr2i18n("Filename", nullptr));
        doNotFilterRecipients->setText(tr2i18n("Do not filter (All recipients are allowed)", nullptr));
        resizeEachEmails->setText(tr2i18n("Resize if each emails address contains one of the pattern", nullptr));
        resizeOneEmails->setText(tr2i18n("Resize if one emails address contains one of the pattern", nullptr));
        resizeEmailsPattern->setPlaceholderText(tr2i18n("Specify patterns separate with ';'", nullptr));
        doNotResizeEachEmails->setText(tr2i18n("Do not resize if each emails address contains one of the pattern", nullptr));
        doNotResizeOneEmails->setText(tr2i18n("Do not resize if one emails address contains one of the pattern", nullptr));
        doNotResizePattern->setPlaceholderText(tr2i18n("Specify patterns separate with ';'", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), tr2i18n("Recipients", nullptr));
        (void)ImageScalingWidget;
    } // retranslateUi

};

namespace Ui {
    class ImageScalingWidget: public Ui_ImageScalingWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // IMAGESCALINGWIDGET_H

