/****************************************************************************
** Meta object code from reading C++ file 'view.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../messagelib-26.07.80/messagelist/src/core/view.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'view.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11MessageList4Core4ViewE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageList::Core::View::qt_create_metaobjectdata<qt_meta_tag_ZN11MessageList4Core4ViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageList::Core::View",
        "slotCollapseAllGroups",
        "",
        "slotExpandAllGroups",
        "slotExpandCurrentItem",
        "slotCollapseCurrentItem",
        "slotExpandAllThreads",
        "slotCollapseAllThreads",
        "slotHeaderContextMenuRequested",
        "QPoint",
        "pnt",
        "slotShowHideColumn",
        "columnIndex",
        "slotAdjustColumnSizes",
        "slotShowDefaultColumns",
        "slotDisplayTooltips",
        "showTooltips",
        "slotHeaderSectionResized",
        "logicalIndex",
        "oldWidth",
        "newWidth",
        "slotSelectionChanged",
        "QItemSelection",
        "current",
        "saveThemeColumnState",
        "applyThemeColumns"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotCollapseAllGroups'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotExpandAllGroups'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotExpandCurrentItem'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCollapseCurrentItem'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotExpandAllThreads'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCollapseAllThreads'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHeaderContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotShowHideColumn'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'slotAdjustColumnSizes'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShowDefaultColumns'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDisplayTooltips'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'slotHeaderSectionResized'
        QtMocHelpers::SlotData<void(int, int, int)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Int, 19 }, { QMetaType::Int, 20 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 22, 23 }, { 0x80000000 | 22, 2 },
        }}),
        // Slot 'saveThemeColumnState'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'applyThemeColumns'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<View, qt_meta_tag_ZN11MessageList4Core4ViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageList::Core::View::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4Core4ViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4Core4ViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11MessageList4Core4ViewE_t>.metaTypes,
    nullptr
} };

void MessageList::Core::View::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<View *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotCollapseAllGroups(); break;
        case 1: _t->slotExpandAllGroups(); break;
        case 2: _t->slotExpandCurrentItem(); break;
        case 3: _t->slotCollapseCurrentItem(); break;
        case 4: _t->slotExpandAllThreads(); break;
        case 5: _t->slotCollapseAllThreads(); break;
        case 6: _t->slotHeaderContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 7: _t->slotShowHideColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->slotAdjustColumnSizes(); break;
        case 9: _t->slotShowDefaultColumns(); break;
        case 10: _t->slotDisplayTooltips((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->slotHeaderSectionResized((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 12: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 13: _t->saveThemeColumnState(); break;
        case 14: _t->applyThemeColumns(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        }
    }
}

const QMetaObject *MessageList::Core::View::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageList::Core::View::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MessageList4Core4ViewE_t>.strings))
        return static_cast<void*>(this);
    return QTreeView::qt_metacast(_clname);
}

int MessageList::Core::View::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
