// This file is generated by kconfig_compiler_kf6 from customtemplates_kfg.kcfg.
// All changes you do to this file will be lost.

#include "customtemplates_kfg.h"

#include <klocalizedstring.h>

using namespace TemplateParser;

CTemplates::CTemplates( const QString & name )
  : KConfigSkeleton( QStringLiteral( "customtemplatesrc" ) )
  , mParamname(name)
{
  setCurrentGroup( QStringLiteral( "CTemplates #%1" ).arg( mParamname ) );

  mContentItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Content" ), mContent );
  mContentItem->setLabel( i18nd("libtemplateparser6", "Template content") );
  addItem( mContentItem, QStringLiteral( "Content" ) );
  mShortcutItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Shortcut" ), mShortcut );
  mShortcutItem->setLabel( i18nd("libtemplateparser6", "Template shortcut") );
  addItem( mShortcutItem, QStringLiteral( "Shortcut" ) );
  mTypeItem = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "Type" ), mType, 0 );
  mTypeItem->setLabel( i18nd("libtemplateparser6", "Template type") );
  addItem( mTypeItem, QStringLiteral( "Type" ) );
  mToItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "To" ), mTo );
  mToItem->setLabel( i18nd("libtemplateparser6", "To") );
  addItem( mToItem, QStringLiteral( "To" ) );
  mCCItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CC" ), mCC );
  mCCItem->setLabel( i18nd("libtemplateparser6", "CC") );
  addItem( mCCItem, QStringLiteral( "CC" ) );
}

CTemplates::~CTemplates()
{
}

