/********************************************************************************
** Form generated from reading UI file 'customtemplates_base.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CUSTOMTEMPLATES_BASE_H
#define UI_CUSTOMTEMPLATES_BASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kkeysequencewidget.h"
#include "templateparseremailaddressrequesterinterfacewidget.h"
#include "templatesinsertcommandpushbutton.h"
#include "templatestextedit.h"

QT_BEGIN_NAMESPACE

class Ui_CustomTemplatesBase
{
public:
    QVBoxLayout *vboxLayout;
    QSplitter *splitter2;
    QWidget *layout9;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout;
    QLineEdit *mName;
    QPushButton *mAdd;
    QPushButton *mRemove;
    QPushButton *mDuplicate;
    QTreeWidget *mList;
    QLabel *mHelp;
    QFrame *mEditFrame;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    QLabel *mToLabel;
    QLabel *mCCLabel;
    QVBoxLayout *verticalLayout;
    TemplateParser::TemplateParserEmailAddressRequesterInterfaceWidget *mToEdit;
    TemplateParser::TemplateParserEmailAddressRequesterInterfaceWidget *mCCEdit;
    TemplateParser::TemplatesTextEdit *mEdit;
    QGridLayout *gridLayout;
    QComboBox *mType;
    QLabel *textLabel1_2;
    QLabel *textLabel1;
    KKeySequenceWidget *mKeySequenceWidget;
    TemplateParser::TemplatesInsertCommandPushButton *mInsertCommand;

    void setupUi(QWidget *CustomTemplatesBase)
    {
        if (CustomTemplatesBase->objectName().isEmpty())
            CustomTemplatesBase->setObjectName("CustomTemplatesBase");
        CustomTemplatesBase->resize(743, 480);
        vboxLayout = new QVBoxLayout(CustomTemplatesBase);
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        splitter2 = new QSplitter(CustomTemplatesBase);
        splitter2->setObjectName("splitter2");
        splitter2->setOrientation(Qt::Horizontal);
        splitter2->setChildrenCollapsible(false);
        layout9 = new QWidget(splitter2);
        layout9->setObjectName("layout9");
        vboxLayout1 = new QVBoxLayout(layout9);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        mName = new QLineEdit(layout9);
        mName->setObjectName("mName");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(mName->sizePolicy().hasHeightForWidth());
        mName->setSizePolicy(sizePolicy);
        mName->setMinimumSize(QSize(100, 0));

        hboxLayout->addWidget(mName);

        mAdd = new QPushButton(layout9);
        mAdd->setObjectName("mAdd");

        hboxLayout->addWidget(mAdd);

        mRemove = new QPushButton(layout9);
        mRemove->setObjectName("mRemove");

        hboxLayout->addWidget(mRemove);

        mDuplicate = new QPushButton(layout9);
        mDuplicate->setObjectName("mDuplicate");

        hboxLayout->addWidget(mDuplicate);


        vboxLayout1->addLayout(hboxLayout);

        mList = new QTreeWidget(layout9);
        mList->setObjectName("mList");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mList->sizePolicy().hasHeightForWidth());
        mList->setSizePolicy(sizePolicy1);
        mList->setRootIsDecorated(false);
        mList->setAllColumnsShowFocus(true);

        vboxLayout1->addWidget(mList);

        mHelp = new QLabel(layout9);
        mHelp->setObjectName("mHelp");

        vboxLayout1->addWidget(mHelp);

        splitter2->addWidget(layout9);
        mEditFrame = new QFrame(splitter2);
        mEditFrame->setObjectName("mEditFrame");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(12);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(mEditFrame->sizePolicy().hasHeightForWidth());
        mEditFrame->setSizePolicy(sizePolicy2);
        mEditFrame->setFrameShape(QFrame::NoFrame);
        mEditFrame->setFrameShadow(QFrame::Raised);
        verticalLayout_3 = new QVBoxLayout(mEditFrame);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        mToLabel = new QLabel(mEditFrame);
        mToLabel->setObjectName("mToLabel");

        verticalLayout_2->addWidget(mToLabel);

        mCCLabel = new QLabel(mEditFrame);
        mCCLabel->setObjectName("mCCLabel");

        verticalLayout_2->addWidget(mCCLabel);


        horizontalLayout->addLayout(verticalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        mToEdit = new TemplateParser::TemplateParserEmailAddressRequesterInterfaceWidget(mEditFrame);
        mToEdit->setObjectName("mToEdit");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(mToEdit->sizePolicy().hasHeightForWidth());
        mToEdit->setSizePolicy(sizePolicy3);

        verticalLayout->addWidget(mToEdit);

        mCCEdit = new TemplateParser::TemplateParserEmailAddressRequesterInterfaceWidget(mEditFrame);
        mCCEdit->setObjectName("mCCEdit");
        sizePolicy3.setHeightForWidth(mCCEdit->sizePolicy().hasHeightForWidth());
        mCCEdit->setSizePolicy(sizePolicy3);

        verticalLayout->addWidget(mCCEdit);


        horizontalLayout->addLayout(verticalLayout);


        verticalLayout_3->addLayout(horizontalLayout);

        mEdit = new TemplateParser::TemplatesTextEdit(mEditFrame);
        mEdit->setObjectName("mEdit");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy4.setHorizontalStretch(3);
        sizePolicy4.setVerticalStretch(1);
        sizePolicy4.setHeightForWidth(mEdit->sizePolicy().hasHeightForWidth());
        mEdit->setSizePolicy(sizePolicy4);

        verticalLayout_3->addWidget(mEdit);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        mType = new QComboBox(mEditFrame);
        mType->addItem(QString());
        mType->addItem(QString());
        mType->addItem(QString());
        mType->addItem(QString());
        mType->setObjectName("mType");

        gridLayout->addWidget(mType, 1, 2, 1, 1);

        textLabel1_2 = new QLabel(mEditFrame);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabel1_2, 0, 1, 1, 1);

        textLabel1 = new QLabel(mEditFrame);
        textLabel1->setObjectName("textLabel1");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy5.setHorizontalStretch(1);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(textLabel1->sizePolicy().hasHeightForWidth());
        textLabel1->setSizePolicy(sizePolicy5);
        textLabel1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabel1, 1, 1, 1, 1);

        mKeySequenceWidget = new KKeySequenceWidget(mEditFrame);
        mKeySequenceWidget->setObjectName("mKeySequenceWidget");

        gridLayout->addWidget(mKeySequenceWidget, 0, 2, 1, 1);

        mInsertCommand = new TemplateParser::TemplatesInsertCommandPushButton(mEditFrame);
        mInsertCommand->setObjectName("mInsertCommand");

        gridLayout->addWidget(mInsertCommand, 0, 0, 1, 1);


        verticalLayout_3->addLayout(gridLayout);

        splitter2->addWidget(mEditFrame);

        vboxLayout->addWidget(splitter2);

#if QT_CONFIG(shortcut)
        textLabel1_2->setBuddy(mKeySequenceWidget);
        textLabel1->setBuddy(mType);
#endif // QT_CONFIG(shortcut)

        retranslateUi(CustomTemplatesBase);

        QMetaObject::connectSlotsByName(CustomTemplatesBase);
    } // setupUi

    void retranslateUi(QWidget *CustomTemplatesBase)
    {
#if QT_CONFIG(tooltip)
        mName->setToolTip(tr2i18n("Enter a name for the custom template", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mName->setWhatsThis(tr2i18n("To create a custom template, first enter a name for that template and press the '+' button.", "@info:whatsthis"));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        mAdd->setToolTip(tr2i18n("Add a new custom template", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mAdd->setWhatsThis(tr2i18n("Press this button if you want to create a new custom template with the specified name", "@info:whatsthis"));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        mRemove->setToolTip(tr2i18n("Remove the selected custom template", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mRemove->setWhatsThis(tr2i18n("Press this button if you want to remove the currently selected custom template", "@info:whatsthis"));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        mDuplicate->setToolTip(tr2i18n("Duplicate the selected custom template", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mDuplicate->setWhatsThis(tr2i18n("Press this button if you want to duplicate the currently selected custom template", "@info:whatsthis"));
#endif // QT_CONFIG(whatsthis)
        QTreeWidgetItem *___qtreewidgetitem = mList->headerItem();
        ___qtreewidgetitem->setText(1, tr2i18n("Name", "@title:column Name of the custom template."));
        ___qtreewidgetitem->setText(0, tr2i18n("Type", "@title:column Type of custom template"));
#if QT_CONFIG(tooltip)
        mList->setToolTip(tr2i18n("List of custom templates available", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mList->setWhatsThis(tr2i18n("This is a list of all the custom templates available.", "@info:whatsthis"));
#endif // QT_CONFIG(whatsthis)
        mToLabel->setText(tr2i18n("To:", "@label:textbox Recipient mail address"));
        mCCLabel->setText(tr2i18n("CC:", "@label"));
#if QT_CONFIG(tooltip)
        mToEdit->setToolTip(tr2i18n("Additional recipients of the message", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mToEdit->setWhatsThis(tr2i18n("When using this template, the default recipients are those you enter here.  This is a comma-separated list of mail addresses.", "@info:whatsthis"));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        mCCEdit->setToolTip(tr2i18n("Additional recipients who get a copy of the message", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mCCEdit->setWhatsThis(tr2i18n("When using this template, the recipients you enter here will by default get a copy of this message.  This is a comma-separated list of mail addresses.", "@info:whatsthis"));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        mEdit->setToolTip(tr2i18n("The custom template", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mEdit->setWhatsThis(tr2i18n("This area contains the associated custom template.", "@info:whatsthis"));
#endif // QT_CONFIG(whatsthis)
        mType->setItemText(0, tr2i18n("Universal", "@item:inlistbox Universal custom template type."));
        mType->setItemText(1, tr2i18n("Reply", "@item:inlistbox Custom template type for message replies"));
        mType->setItemText(2, tr2i18n("Reply to All", "@item:inlistbox Custom template type for reply-to-all messages"));
        mType->setItemText(3, tr2i18n("Forward", "@item:inlistbox Custom template type for message forwards"));

        textLabel1_2->setText(tr2i18n("Shortc&ut:", "@label"));
        textLabel1->setText(tr2i18n("&Template type:", "@label"));
        (void)CustomTemplatesBase;
    } // retranslateUi

};

namespace Ui {
    class CustomTemplatesBase: public Ui_CustomTemplatesBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CUSTOMTEMPLATES_BASE_H

